@@ -1,8 +1,8 @@
 /*
  *  Hamlib Alinco backend - main file
- *  Copyright (c) 2001-2003 by Stephane Fillod
+ *  Copyright (c) 2001-2005 by Stephane Fillod
  *
- *	$Id: alinco.c,v 1.22 2004/03/15 04:02:08 nj8j Exp $
+ *	$Id: alinco.c,v 1.26 2005/01/25 00:19:38 fillods Exp $
  *
  *   This library is free software; you can redistribute it and/or modify
  *   it under the terms of the GNU Library General Public License as
@@ -214,7 +214,7 @@ int alinco_set_freq(RIG *rig, vfo_t vfo, freq_t freq)
 				return -RIG_EINVAL;
 
 		/* at least 6 digits */
-		freq_len = sprintf(freqbuf, AL CMD_RXFREQ "%06Ld" EOM, (long long)freq);
+		freq_len = sprintf(freqbuf, AL CMD_RXFREQ "%06"PRIll EOM, (long long)freq);
 
 		return alinco_transaction (rig, freqbuf, freq_len, NULL, NULL);
 }
@@ -254,7 +254,7 @@ int alinco_get_freq(RIG *rig, vfo_t vfo, freq_t *freq)
 
 		/* extract RX freq */
 		freqbuf[16] = '\0';
-		sscanf(freqbuf+6, "%"FREQFMT, freq);
+		sscanf(freqbuf+6, "%"SCNfreq, freq);
 
 		return RIG_OK;
 }
@@ -409,7 +409,7 @@ int alinco_set_split_freq(RIG *rig, vfo_t vfo, freq_t tx_freq)
 				return -RIG_EINVAL;
 
 		/* at least 6 digits */
-		freq_len = sprintf(freqbuf, AL CMD_TXFREQ "%06Ld" EOM, (long long)tx_freq);
+		freq_len = sprintf(freqbuf, AL CMD_TXFREQ "%06"PRIll EOM, (long long)tx_freq);
 
 		retval = alinco_transaction (rig, freqbuf, freq_len, NULL, NULL);
 
@@ -431,7 +431,7 @@ int alinco_get_split_freq(RIG *rig, vfo_t vfo, freq_t *tx_freq)
 
 		/* extract TX freq first, as RX kills freqbuf[16] */
 		freqbuf[26] = '\0';
-		sscanf(freqbuf+16, "%"FREQFMT, tx_freq);
+		sscanf(freqbuf+16, "%"SCNfreq, tx_freq);
 
 		return RIG_OK;
 }
@@ -604,6 +604,53 @@ int alinco_set_level(RIG *rig, vfo_t vfo, setting_t level, value_t val)
 		case RIG_LEVEL_RFPOWER:
 			cmd_len = sprintf(cmdbuf, AL CMD_PWR "%1d" EOM, val.f<0.5 ? 1 : 0);
 
+			return alinco_transaction (rig, cmdbuf, cmd_len, NULL, NULL);
+		case RIG_LEVEL_KEYSPD:
+			if (val.i < 6) {
+			    lvl = 31;
+			} else if(val.i >= 6 && val.i < 20) {
+			    lvl = val.i + 25;
+			} else if(val.i >= 20 && val.i <= 50) {
+			    lvl = val.i - 20;
+			} else {
+			    lvl = 30;
+			}
+			cmd_len = sprintf(cmdbuf, AL CMD_SDATA "P%02d" EOM, lvl);
+
+			return alinco_transaction (rig, cmdbuf, cmd_len, NULL, NULL);
+
+		case RIG_LEVEL_CWPITCH:
+		    	if(val.i < 426) {
+			    lvl = 5;
+			} else if(val.i >= 426 && val.i <= 475) {
+			    lvl = 6;
+			} else if(val.i >= 476 && val.i <= 525) {
+			    lvl = 7;
+			} else if(val.i >= 526 && val.i <= 575) {
+			    lvl = 8;
+			} else if(val.i >= 576 && val.i <= 625) {
+			    lvl = 9;
+			} else if(val.i >= 626 && val.i <= 675) {
+			    lvl = 10;
+			} else if(val.i >= 676 && val.i <= 725) {
+			    lvl = 11;
+			} else if(val.i >= 726 && val.i <= 775) {
+			    lvl = 12;
+			} else if(val.i >= 776 && val.i <= 825) {
+			    lvl = 0;
+			} else if(val.i >= 826 && val.i <= 875) {
+			    lvl = 1;
+			} else if(val.i >= 876 && val.i <= 925) {
+			    lvl = 2;
+			} else if(val.i >= 926 && val.i <= 975) {
+			    lvl = 3;
+			} else if(val.i >= 976 && val.i <= 1025) {
+			    lvl = 4;
+			} else {
+			    lvl = 4;
+		        }
+			cmd_len = sprintf(cmdbuf, AL CMD_SDATA "M%02d" EOM, lvl);
+
 			return alinco_transaction (rig, cmdbuf, cmd_len, NULL, NULL);
 
 		default:
@@ -611,6 +658,7 @@ int alinco_set_level(RIG *rig, vfo_t vfo, setting_t level, value_t val)
 			return -RIG_EINVAL;
 		}
 
+
 		return RIG_OK;
 }
 
@@ -645,9 +693,6 @@ int alinco_get_level(RIG *rig, vfo_t vfo, setting_t level, value_t *val)
 			val->i = atoi(lvlbuf+3);
 			break;
 
-		case RIG_LEVEL_SQLSTAT:
-			return -RIG_ENIMPL;	/* get_dcd ? */
-
 		case RIG_LEVEL_PREAMP:
 			retval = current_data_read(rig, lvlbuf);
 			if (retval != RIG_OK)
