@@ -2,7 +2,7 @@
  *  Hamlib CI-V backend - main file
  *  Copyright (c) 2000-2004 by Stephane Fillod
  *
- *	$Id: icom.c,v 1.86 2004/05/18 06:55:05 fillods Exp $
+ *	$Id: icom.c,v 1.92 2004/09/26 08:35:03 fillods Exp $
  *
  *   This library is free software; you can redistribute it and/or modify
  *   it under the terms of the GNU Library General Public License as
@@ -232,9 +232,11 @@ static const struct icom_addr icom_addr_list[] = {
 		{ RIG_MODEL_IC756, 0x50 },
 		{ RIG_MODEL_IC756PRO, 0x5c },
 		{ RIG_MODEL_IC756PROII, 0x64 },
+		{ RIG_MODEL_IC756PROIII, 0x6e },
 		{ RIG_MODEL_IC761, 0x1e },
 		{ RIG_MODEL_IC765, 0x2c },
 		{ RIG_MODEL_IC775, 0x46 },
+		{ RIG_MODEL_IC7800, 0x6a },
 		{ RIG_MODEL_IC781, 0x26 },
 		{ RIG_MODEL_IC820, 0x42 },
 		{ RIG_MODEL_IC821, 0x4c },
@@ -244,6 +246,7 @@ static const struct icom_addr icom_addr_list[] = {
 		{ RIG_MODEL_IC1271, 0x24 },
 		{ RIG_MODEL_IC1275, 0x18 },
 		{ RIG_MODEL_ICR10, 0x52 },
+		{ RIG_MODEL_ICR20, 0x6c },
 		{ RIG_MODEL_ICR71, 0x1a },
 		{ RIG_MODEL_ICR72, 0x32 },
 		{ RIG_MODEL_ICR75, 0x5a },
@@ -452,7 +455,7 @@ int icom_set_mode(RIG *rig, vfo_t vfo, rmode_t mode, pbwidth_t width)
 		struct icom_priv_data *priv;
 		struct rig_state *rs;
 		unsigned char ackbuf[MAXFRAMELEN];
-		unsigned char icmode; 
+		unsigned char icmode;
 		signed char icmode_ext;
 		int ack_len, retval, err;
 
@@ -608,6 +611,23 @@ int icom_set_level(RIG *rig, vfo_t vfo, setting_t level, value_t val)
 		else
 				icom_val = val.i;
 
+		/* convert values to 0 .. 255 range */
+		if (rig->caps->rig_model == RIG_MODEL_ICR75) {
+			switch (level) {
+				case RIG_LEVEL_NR:
+					icom_val = val.f * 240;
+					break;
+				case RIG_LEVEL_PBT_IN:
+				case RIG_LEVEL_PBT_OUT:
+					icom_val = (val.f / 10.0) + 128;
+					if (icom_val > 255)
+						icom_val = 255;
+					break;
+				default:
+					break;
+			}
+		}
+
 		/*
 		 * Most of the time, the data field is a 3 digit BCD,
 		 * but in *big endian* order: 0000..0255
@@ -680,6 +700,14 @@ int icom_set_level(RIG *rig, vfo_t vfo, setting_t level, value_t val)
 		case RIG_LEVEL_CWPITCH:
 			lvl_cn = C_CTL_LVL;
 			lvl_sc = S_LVL_CWPITCH;
+			/* use 'set mode' call for CWPITCH on IC-R75*/
+			if (rig->caps->rig_model == RIG_MODEL_ICR75) {
+				lvl_cn = C_CTL_MEM;
+				lvl_sc = S_MEM_MODE_SLCT;
+				lvl_len = 3;
+				lvlbuf[0] = S_PRM_CWPITCH;
+				to_bcd_be(lvlbuf+1, (long long)icom_val, 4);
+			}
 			break;
 		case RIG_LEVEL_RFPOWER:
 			lvl_cn = C_CTL_LVL;
@@ -704,6 +732,10 @@ int icom_set_level(RIG *rig, vfo_t vfo, setting_t level, value_t val)
 		case RIG_LEVEL_AGC:
 			lvl_cn = C_CTL_FUNC;
 			lvl_sc = S_FUNC_AGC;
+			if (rig->caps->rig_model == RIG_MODEL_ICR75) {
+				lvl_len = 1;
+				lvlbuf[0] = val.i;
+			}
 			break;
 		case RIG_LEVEL_BKINDL:
 			lvl_cn = C_CTL_LVL;
@@ -752,8 +784,8 @@ int icom_get_level(RIG *rig, vfo_t vfo, setting_t level, value_t *val)
 {
 		struct icom_priv_data *priv;
 		struct rig_state *rs;
-		unsigned char lvlbuf[MAXFRAMELEN];
-		int lvl_len;
+		unsigned char lvlbuf[MAXFRAMELEN], lvl2buf[MAXFRAMELEN];
+		int lvl_len, lvl2_len;
 		int lvl_cn, lvl_sc;		/* Command Number, Subcommand */
 		int icom_val;
 		int cmdhead;
@@ -762,20 +794,16 @@ int icom_get_level(RIG *rig, vfo_t vfo, setting_t level, value_t *val)
 		rs = &rig->state;
 		priv = (struct icom_priv_data*)rs->priv;
 
-
+		lvl2_len = 0;
 		/* Optimize:
 		 *   sort the switch cases with the most frequent first
 		 */
 		switch (level) {
+		case RIG_LEVEL_STRENGTH:
 		case RIG_LEVEL_RAWSTR:
 			lvl_cn = C_RD_SQSM;
 			lvl_sc = S_SML;
 			break;
-		case RIG_LEVEL_SQLSTAT:
-			lvl_cn = C_RD_SQSM;
-			lvl_sc = S_SQL;
-			break;
-
 		case RIG_LEVEL_PREAMP:
 			lvl_cn = C_CTL_FUNC;
 			lvl_sc = S_FUNC_PAMP;
@@ -819,6 +847,13 @@ int icom_get_level(RIG *rig, vfo_t vfo, setting_t level, value_t *val)
 		case RIG_LEVEL_CWPITCH:
 			lvl_cn = C_CTL_LVL;
 			lvl_sc = S_LVL_CWPITCH;
+			/* use 'set mode' call for CWPITCH on IC-R75*/
+			if (rig->caps->rig_model == RIG_MODEL_ICR75) {
+				lvl_cn = C_CTL_MEM;
+				lvl_sc = S_MEM_MODE_SLCT;
+				lvl2_len = 1;
+				lvl2buf[0] = S_PRM_CWPITCH;
+			}
 			break;
 		case RIG_LEVEL_RFPOWER:
 			lvl_cn = C_CTL_LVL;
@@ -869,7 +904,8 @@ int icom_get_level(RIG *rig, vfo_t vfo, setting_t level, value_t *val)
 			return -RIG_EINVAL;
 		}
 
-		retval = icom_transaction (rig, lvl_cn, lvl_sc, NULL, 0,
+		/* use lvl2buf and lvl2_len for 'set mode' subcommand */
+		retval = icom_transaction (rig, lvl_cn, lvl_sc, lvl2buf, lvl2_len,
 						lvlbuf, &lvl_len);
 		if (retval != RIG_OK)
 				return retval;
@@ -879,6 +915,11 @@ int icom_get_level(RIG *rig, vfo_t vfo, setting_t level, value_t *val)
 		 */
 		cmdhead = (lvl_sc == -1) ? 1:2;
 		lvl_len -= cmdhead;
+		/* back off one char since first char in buffer is now 'set mode' subcommand */
+		if ((rig->caps->rig_model == RIG_MODEL_ICR75)&&(level==RIG_LEVEL_CWPITCH)){
+			cmdhead = 3;
+			lvl_len--;
+		}
 
 		if (lvlbuf[0] != ACK && lvlbuf[0] != lvl_cn) {
 				rig_debug(RIG_DEBUG_ERR,"icom_get_level: ack NG (%#.2x), "
@@ -892,18 +933,13 @@ int icom_get_level(RIG *rig, vfo_t vfo, setting_t level, value_t *val)
 		 */
 		icom_val = from_bcd_be(lvlbuf+cmdhead, lvl_len*2);
 
-
 		switch (level) {
 		case RIG_LEVEL_RAWSTR:
 			val->i = icom_val;
 			break;
-		case RIG_LEVEL_SQLSTAT:
-			/*
-			 * 0x00=sql closed, 0x01=sql open
-			 */
-			val->i = icom_val;
+		case RIG_LEVEL_STRENGTH:
+			val->i = (int)rig_raw2val(icom_val,&rig->caps->str_cal);
 			break;
-
 		case RIG_LEVEL_PREAMP:
 			if (icom_val == 0) {
 					val->i = 0;
@@ -924,6 +960,24 @@ int icom_get_level(RIG *rig, vfo_t vfo, setting_t level, value_t *val)
 				val->i = icom_val;
 		}
 
+		/* convert values from 0 .. 255 range */
+		if (rig->caps->rig_model == RIG_MODEL_ICR75) {
+			switch (level) {
+				case RIG_LEVEL_NR:
+					val->f = (float)icom_val / 240;
+					break;
+				case RIG_LEVEL_PBT_IN:
+				case RIG_LEVEL_PBT_OUT:
+					if (icom_val == 255)
+						val->f = 1280.0;
+					else
+						val->f = (float)(icom_val - 128) * 10.0;
+					break;
+				default:
+					break;
+			}
+		}
+
 		rig_debug(RIG_DEBUG_TRACE,"icom_get_level: %d %d %d %f\n", lvl_len,
 						icom_val, val->i, val->f);
 
@@ -992,15 +1046,15 @@ int icom_set_ptt(RIG *rig, vfo_t vfo, ptt_t ptt)
 {
 		struct icom_priv_data *priv;
 		struct rig_state *rs;
-		unsigned char ackbuf[MAXFRAMELEN], ptt_sc;
+		unsigned char ackbuf[MAXFRAMELEN], pttbuf[1];
 		int ack_len, retval;
 
 		rs = &rig->state;
 		priv = (struct icom_priv_data*)rs->priv;
 
-		ptt_sc = ptt == RIG_PTT_ON ? S_PTT_ON:S_PTT_OFF;
+		pttbuf[0] = ptt == RIG_PTT_ON ? 1 : 0;
 
-		retval = icom_transaction (rig, C_CTL_PTT, ptt_sc, NULL, 0,
+		retval = icom_transaction (rig, C_CTL_PTT, S_PTT, pttbuf, 1,
 						ackbuf, &ack_len);
 		if (retval != RIG_OK)
 				return retval;
@@ -1028,7 +1082,7 @@ int icom_get_ptt(RIG *rig, vfo_t vfo, ptt_t *ptt)
 		rs = &rig->state;
 		priv = (struct icom_priv_data*)rs->priv;
 
-		retval = icom_transaction (rig, C_CTL_PTT, -1, NULL, 0,
+		retval = icom_transaction (rig, C_CTL_PTT, S_PTT, NULL, 0,
 						pttbuf, &ptt_len);
 		if (retval != RIG_OK)
 				return retval;
@@ -1043,7 +1097,7 @@ int icom_get_ptt(RIG *rig, vfo_t vfo, ptt_t *ptt)
 				return -RIG_ERJCTED;
 		}
 
-		*ptt = pttbuf[1] == S_PTT_ON ? RIG_PTT_ON : RIG_PTT_OFF;
+		*ptt = pttbuf[1] == 1 ? RIG_PTT_ON : RIG_PTT_OFF;
 
 		return RIG_OK;
 }
@@ -1837,6 +1891,185 @@ int icom_get_func(RIG *rig, vfo_t vfo, setting_t func, int *status)
 		return RIG_OK;
 }
 
+/*
+ * icom_set_parm
+ * Assumes rig!=NULL
+ */
+int icom_set_parm(RIG *rig, setting_t parm, value_t val)
+{
+		unsigned char prmbuf[MAXFRAMELEN], ackbuf[MAXFRAMELEN];
+		int ack_len, prm_len;
+		int prm_cn, prm_sc;
+		int icom_val;
+		int retval;
+		int min,hr,sec;
+
+		switch (parm) {
+		case RIG_PARM_ANN:
+			if ((val.i == RIG_ANN_FREQ) || (val.i == RIG_ANN_RXMODE)) {
+				prm_cn = C_CTL_ANN;
+				prm_sc = val.i;
+				prm_len = 0;
+			}
+			else {
+				if ((val.i == RIG_ANN_ENG)||(val.i == RIG_ANN_JAP)) {
+					prm_cn = C_CTL_MEM;
+					prm_sc = S_MEM_MODE_SLCT;
+					prm_len = 2;
+					prmbuf[0] = S_PRM_LANG;
+					prmbuf[1] = (val.i == RIG_ANN_ENG ? 0 : 1);
+				}
+				else {
+					rig_debug(RIG_DEBUG_ERR,"Unsupported set_parm_ann %d\n", val.i);
+					return -RIG_EINVAL;
+				}
+			}
+			break;
+		case RIG_PARM_APO:
+			prm_cn = C_CTL_MEM;
+			prm_sc = S_MEM_MODE_SLCT;
+			hr = (float)val.i/60.0;
+			min = val.i - (hr*60);
+			prm_len = 3;
+			prmbuf[0] = S_PRM_SLPTM;
+			to_bcd_be(prmbuf+1, (long long)hr, 2);
+			to_bcd_be(prmbuf+2, (long long)min, 2);
+			break;
+		case RIG_PARM_BACKLIGHT:
+			prm_cn = C_CTL_MEM;
+			prm_sc = S_MEM_MODE_SLCT;
+			icom_val = val.f * 255;
+			prm_len = 3;
+			prmbuf[0] = S_PRM_BACKLT;
+			to_bcd_be(prmbuf+1, (long long)icom_val, (prm_len-1)*2);
+			break;
+		case RIG_PARM_BEEP:
+			prm_cn = C_CTL_MEM;
+			prm_sc = S_MEM_MODE_SLCT;
+			prm_len = 2;
+			prmbuf[0] = S_PRM_BEEP;
+			prmbuf[1] = val.i;
+			break;
+		case RIG_PARM_TIME:
+			prm_cn = C_CTL_MEM;
+			prm_sc = S_MEM_MODE_SLCT;
+			hr = (float)val.i/3600.0;
+			min = (float)(val.i - (hr*3600))/60.0;
+			sec = (val.i - (hr*3600) - (min*60));
+			prm_len = 4;
+			prmbuf[0] = S_PRM_TIME;
+			to_bcd_be(prmbuf+1, (long long)hr, 2);
+			to_bcd_be(prmbuf+2, (long long)min, 2);
+			to_bcd_be(prmbuf+3, (long long)sec, 2);
+			break;
+		default:
+		  rig_debug(RIG_DEBUG_ERR,"Unsupported set_parm %d\n", parm);
+		  return -RIG_EINVAL;
+		}
+
+		retval = icom_transaction(rig, prm_cn, prm_sc, prmbuf, prm_len,
+						ackbuf, &ack_len);
+		if (retval != RIG_OK)
+				return retval;
+
+		if (ack_len != 1) {
+				rig_debug(RIG_DEBUG_ERR,"icom_set_parm: wrong frame len=%d\n",
+								ack_len);
+				return -RIG_EPROTO;
+		}
+
+		return RIG_OK;
+}
+
+/*
+ * icom_get_parm
+ * Assumes rig!=NULL
+ */
+int icom_get_parm(RIG *rig, setting_t parm, value_t *val)
+{
+		unsigned char prmbuf[MAXFRAMELEN], resbuf[MAXFRAMELEN];
+		int prm_len, res_len;
+		int prm_cn, prm_sc;
+		int icom_val;
+		int cmdhead;
+		int retval;
+		int min,hr,sec;
+
+		switch (parm) {
+		case RIG_PARM_APO:
+			prm_cn = C_CTL_MEM;
+			prm_sc = S_MEM_MODE_SLCT;
+			prm_len = 1;
+			prmbuf[0] = S_PRM_SLPTM;
+			break;
+		case RIG_PARM_BACKLIGHT:
+			prm_cn = C_CTL_MEM;
+			prm_sc = S_MEM_MODE_SLCT;
+			prm_len = 1;
+			prmbuf[0] = S_PRM_BACKLT;
+			break;
+		case RIG_PARM_BEEP:
+			prm_cn = C_CTL_MEM;
+			prm_sc = S_MEM_MODE_SLCT;
+			prm_len = 1;
+			prmbuf[0] = S_PRM_BEEP;
+			break;
+		case RIG_PARM_TIME:
+			prm_cn = C_CTL_MEM;
+			prm_sc = S_MEM_MODE_SLCT;
+			prm_len = 1;
+			prmbuf[0] = S_PRM_TIME;
+			break;
+		default:
+			rig_debug(RIG_DEBUG_ERR,"Unsupported get_parm %d", parm);
+			return -RIG_EINVAL;
+		}
+
+		retval = icom_transaction (rig, prm_cn, prm_sc, prmbuf, prm_len,
+						resbuf, &res_len);
+		if (retval != RIG_OK)
+				return retval;
+
+		/*
+		 * strbuf should contain Cn,Sc,[pn],Data area
+		 */
+		cmdhead = (prm_sc == -1) ? 1:3;
+		res_len -= cmdhead;
+
+		if (resbuf[0] != ACK && resbuf[0] != prm_cn) {
+				rig_debug(RIG_DEBUG_ERR,"%s: ack NG (%#.2x), "
+							"len=%d\n", __FUNCTION__,resbuf[0],res_len);
+				return -RIG_ERJCTED;
+		}
+
+		switch (parm) {
+		case RIG_PARM_APO:
+			hr = from_bcd_be(resbuf+cmdhead, 2);
+			min = from_bcd_be(resbuf+cmdhead+1, 2);
+			icom_val = (hr*60)+min;
+			val->i = icom_val;
+			break;
+		case RIG_PARM_TIME:
+			hr = from_bcd_be(resbuf+cmdhead, 2);
+			min = from_bcd_be(resbuf+cmdhead+1, 2);
+			sec = from_bcd_be(resbuf+cmdhead+2, 2);
+			icom_val = (hr*3600)+(min*60)+sec;
+			val->i = icom_val;
+			break;
+		default:
+			icom_val = from_bcd_be(resbuf+cmdhead, res_len*2);
+			if (RIG_PARM_IS_FLOAT(parm))
+				val->f = (float)icom_val/255;
+			else
+				val->i = icom_val;
+		}
+
+		rig_debug(RIG_DEBUG_TRACE,"%s: %d %d %d %f\n",
+				__FUNCTION__, res_len, icom_val, val->i, val->f);
+
+		return RIG_OK;
+}
+
 /*
  * icom_set_ctcss_tone
  * Assumes rig!=NULL, rig->state.priv!=NULL
@@ -2051,21 +2284,36 @@ int icom_set_powerstat(RIG *rig, powerstat_t status)
  */
 int icom_get_powerstat(RIG *rig, powerstat_t *status)
 {
-		unsigned char ackbuf[MAXFRAMELEN];
-		int ack_len, retval;
+		unsigned char cmdbuf[MAXFRAMELEN], ackbuf[MAXFRAMELEN];
+		int cmd_len, ack_len, retval;
 
-		retval = icom_transaction(rig, C_SET_PWR, -1, NULL, 0,
-						ackbuf, &ack_len);
-		if (retval != RIG_OK)
+		/* r75 has no way to get power status, so fake it */
+		if (rig->caps->rig_model == RIG_MODEL_ICR75) {
+			/* getting the mode doesn't work if a memory is blank */
+			/* so use one of the more innculous 'set mode' commands instead */
+			cmd_len = 1;
+			cmdbuf[0] = S_PRM_TIME;
+			retval = icom_transaction(rig, C_CTL_MEM, S_MEM_MODE_SLCT, 
+									cmdbuf, cmd_len, ackbuf, &ack_len);
+			if (retval != RIG_OK)
 				return retval;
 
-		if (ack_len != 1 || ackbuf[0] != ACK) {
-				rig_debug(RIG_DEBUG_ERR,"icom_get_powerstat: ack NG (%#.2x), "
-								"len=%d\n", ackbuf[0],ack_len);
-				return -RIG_ERJCTED;
+			*status = ((ack_len == 6)&&(ackbuf[0] == C_CTL_MEM)) ?
+						RIG_POWER_ON : RIG_POWER_OFF;
 		}
-		*status = ackbuf[1] == S_PWR_ON ? RIG_POWER_ON : RIG_POWER_OFF;
+		else {
+			retval = icom_transaction(rig, C_SET_PWR, -1, NULL, 0,
+							ackbuf, &ack_len);
+			if (retval != RIG_OK)
+					return retval;
 
+			if (ack_len != 1 || ackbuf[0] != ACK) {
+					rig_debug(RIG_DEBUG_ERR,"icom_get_powerstat: ack NG (%#.2x), "
+									"len=%d\n", ackbuf[0],ack_len);
+					return -RIG_ERJCTED;
+			}
+			*status = ackbuf[1] == S_PWR_ON ? RIG_POWER_ON : RIG_POWER_OFF;
+		}
 		return RIG_OK;
 }
 
@@ -2144,6 +2392,7 @@ int icom_set_ant(RIG * rig, vfo_t vfo, ant_t ant)
 		unsigned char antarg;
 		unsigned char ackbuf[MAXFRAMELEN];
 		int ack_len, retval, i_ant;
+		int ant_len;
 
 		rs = &rig->state;
 		priv = (struct icom_priv_data*)rs->priv;
@@ -2154,8 +2403,9 @@ int icom_set_ant(RIG * rig, vfo_t vfo, ant_t ant)
 		i_ant = ant == RIG_ANT_1 ? 0: 1;
 
 		antarg = 0;
+		ant_len = (rig->caps->rig_model == RIG_MODEL_ICR75) ? 0 : 1;
 		retval = icom_transaction (rig, C_CTL_ANT, i_ant,
-						&antarg, 1, ackbuf, &ack_len);
+						&antarg, ant_len, ackbuf, &ack_len);
 		if (retval != RIG_OK)
 				return retval;
 
@@ -2182,6 +2432,14 @@ int icom_get_ant(RIG *rig, vfo_t vfo, ant_t *ant)
 		if (retval != RIG_OK)
 				return retval;
 
+		if (rig->caps->rig_model == RIG_MODEL_ICR75) {
+			if (ack_len != 2 || ackbuf[0] != C_CTL_ANT) {
+				rig_debug(RIG_DEBUG_ERR,"icom_get_ant: ack NG (%#.2x), "
+								"len=%d\n", ackbuf[0],ack_len);
+				return -RIG_ERJCTED;
+			}
+		}
+		else
 		if (ack_len != 1 || ackbuf[0] != ACK) {
 				rig_debug(RIG_DEBUG_ERR,"icom_get_ant: ack NG (%#.2x), "
 								"len=%d\n", ackbuf[0],ack_len);
@@ -2459,8 +2717,8 @@ DECLARE_PROBERIG_BACKEND(icom)
 		/*
 		 * try all possible addresses on the CI-V bus
 		 * FIXME: actualy, old rigs do not support C_RD_TRXID cmd!
-		 * 		Try to be smart, and deduce model depending 
-		 * 		on freq range, return address, and 
+		 * 		Try to be smart, and deduce model depending
+		 * 		on freq range, return address, and
 		 * 		available commands.
 		 */
 		for (civ_addr=0x01; civ_addr<=0x7f; civ_addr++) {
@@ -2605,7 +2863,10 @@ DECLARE_INITRIG_BACKEND(icom)
 	rig_register(&ic756_caps);
 	rig_register(&ic756pro_caps);
 	rig_register(&ic756pro2_caps);
+	rig_register(&ic756pro3_caps);
 	rig_register(&ic765_caps);
+	rig_register(&ic78_caps);
+	rig_register(&ic7800_caps);
 	rig_register(&ic781_caps);
 	rig_register(&ic707_caps);
 	rig_register(&ic728_caps);
@@ -2615,6 +2876,7 @@ DECLARE_INITRIG_BACKEND(icom)
 	rig_register(&ic970_caps);
 
         rig_register(&icr10_caps);
+        rig_register(&icr20_caps);
         rig_register(&icr71_caps);
         rig_register(&icr72_caps);
         rig_register(&icr75_caps);
