@@ -1,8 +1,8 @@
 /*
  *  Hamlib Interface - list of known rigs
- *  Copyright (c) 2000-2004 by Stephane Fillod and Frank Singleton
+ *  Copyright (c) 2000-2005 by Stephane Fillod and Frank Singleton
  *
- *	$Id: riglist.h,v 1.45 2004/04/16 20:01:06 fillods Exp $
+ *	$Id: riglist.h,v 1.55 2005/04/03 19:19:00 fillods Exp $
  *
  *   This library is free software; you can redistribute it and/or modify
  *   it under the terms of the GNU Library General Public License as
@@ -63,7 +63,7 @@
 #define RIG_MODEL_FT847 RIG_MAKE_MODEL(RIG_YAESU, 1)
 #define RIG_MODEL_FT1000 RIG_MAKE_MODEL(RIG_YAESU, 2)
 #define RIG_MODEL_FT1000D RIG_MAKE_MODEL(RIG_YAESU, 3)
-#define RIG_MODEL_FT1000MP RIG_MAKE_MODEL(RIG_YAESU, 4)
+#define RIG_MODEL_FT1000MPMKV RIG_MAKE_MODEL(RIG_YAESU, 4)
 #define RIG_MODEL_FT747 RIG_MAKE_MODEL(RIG_YAESU, 5)
 #define RIG_MODEL_FT757 RIG_MAKE_MODEL(RIG_YAESU, 6)
 #define RIG_MODEL_FT757GXII RIG_MAKE_MODEL(RIG_YAESU, 7)
@@ -83,6 +83,9 @@
 #define RIG_MODEL_FT100 RIG_MAKE_MODEL(RIG_YAESU, 21)
 #define RIG_MODEL_FT857 RIG_MAKE_MODEL(RIG_YAESU, 22)
 #define RIG_MODEL_FT897 RIG_MAKE_MODEL(RIG_YAESU, 23)
+#define RIG_MODEL_FT1000MP RIG_MAKE_MODEL(RIG_YAESU, 24)
+#define RIG_MODEL_FT1000MPMKVFLD RIG_MAKE_MODEL(RIG_YAESU, 25)
+#define RIG_MODEL_VR5000 RIG_MAKE_MODEL(RIG_YAESU, 26)
 
 	/*
 	 * Kenwood
@@ -116,13 +119,13 @@
 #define RIG_MODEL_TS140S RIG_MAKE_MODEL(RIG_KENWOOD, 25)
 #define RIG_MODEL_TMD700 RIG_MAKE_MODEL(RIG_KENWOOD, 26)
 #define RIG_MODEL_TMV7 RIG_MAKE_MODEL(RIG_KENWOOD, 27)
+#define RIG_MODEL_TS480 RIG_MAKE_MODEL(RIG_KENWOOD, 28)
 
 	/*
 	 * Icom
 	 */
 #define RIG_ICOM 3
 #define RIG_BACKEND_ICOM "icom"
-#define RIG_MODEL_ICALL RIG_MAKE_MODEL(RIG_ICOM, 0)	/* deprecated */
 #define RIG_MODEL_IC1271 RIG_MAKE_MODEL(RIG_ICOM, 1)
 #define RIG_MODEL_IC1275 RIG_MAKE_MODEL(RIG_ICOM, 2)
 #define RIG_MODEL_IC271 RIG_MAKE_MODEL(RIG_ICOM, 3)
@@ -172,6 +175,10 @@
 #define RIG_MODEL_IC756PROII RIG_MAKE_MODEL(RIG_ICOM, 47)
 #define RIG_MODEL_ICID1 RIG_MAKE_MODEL(RIG_ICOM, 54)
 #define RIG_MODEL_IC703 RIG_MAKE_MODEL(RIG_ICOM, 55)
+#define RIG_MODEL_IC7800 RIG_MAKE_MODEL(RIG_ICOM, 56)
+#define RIG_MODEL_IC756PROIII RIG_MAKE_MODEL(RIG_ICOM, 57)
+#define RIG_MODEL_ICR20 RIG_MAKE_MODEL(RIG_ICOM, 58)
+/* next one is 60 */
 
 	/*
 	 * Optoelectronics (CI-V)
@@ -186,6 +193,7 @@
 	 */
 #define RIG_MODEL_OMNIVI RIG_MAKE_MODEL(RIG_ICOM, 50)
 #define RIG_MODEL_OMNIVIP RIG_MAKE_MODEL(RIG_ICOM, 51) /* OMNI-VI+ */
+#define RIG_MODEL_PARAGON2 RIG_MAKE_MODEL(RIG_ICOM, 59)
 
 	/*
 	 * Icom PCR
@@ -211,6 +219,7 @@
 #define RIG_MODEL_AR2500 RIG_MAKE_MODEL(RIG_AOR, 9)
 #define RIG_MODEL_AR16 RIG_MAKE_MODEL(RIG_AOR, 10)
 #define RIG_MODEL_SDU5500 RIG_MAKE_MODEL(RIG_AOR, 11)
+#define RIG_MODEL_SDU5000 RIG_MAKE_MODEL(RIG_AOR, 12)
 
 	/*
 	 * JRC
@@ -275,6 +284,7 @@
 #define RIG_MODEL_RA3790 RIG_MAKE_MODEL(RIG_RACAL, 1)
 #define RIG_MODEL_RA3720 RIG_MAKE_MODEL(RIG_RACAL, 2)
 #define RIG_MODEL_RA6790 RIG_MAKE_MODEL(RIG_RACAL, 3)
+#define RIG_MODEL_RA3710 RIG_MAKE_MODEL(RIG_RACAL, 4)
 
 	/*
 	 * Watkins-Johnson
@@ -284,6 +294,7 @@
 #define RIG_MODEL_HF1000 RIG_MAKE_MODEL(RIG_WJ, 1)
 #define RIG_MODEL_HF1000A RIG_MAKE_MODEL(RIG_WJ, 2)
 #define RIG_MODEL_WJ8711 RIG_MAKE_MODEL(RIG_WJ, 3)
+#define RIG_MODEL_WJ8888 RIG_MAKE_MODEL(RIG_WJ, 4)
 
 	/*
 	 * Rohde & Schwarz
@@ -332,6 +343,7 @@
 #define RIG_MODEL_TT526 RIG_MAKE_MODEL(RIG_TENTEC, 6)	/* 6N2 */
 #define RIG_MODEL_TT516 RIG_MAKE_MODEL(RIG_TENTEC, 7)	/* Argonaut V */
 #define RIG_MODEL_TT565 RIG_MAKE_MODEL(RIG_TENTEC, 8)	/* Orion */
+#define RIG_MODEL_TT585 RIG_MAKE_MODEL(RIG_TENTEC, 9)	/* Paragon */
 
 	/*
 	 * Alinco
@@ -377,6 +389,7 @@
 #define RIG_BACKEND_MICROTUNE "microtune"
 #define RIG_MODEL_MICROTUNE_4937 RIG_MAKE_MODEL(RIG_MICROTUNE, 1)	/* eval board */
 #define RIG_MODEL_MICROTUNE_4702 RIG_MAKE_MODEL(RIG_MICROTUNE, 2)	/* Alan's */
+#define RIG_MODEL_MICROTUNE_4707 RIG_MAKE_MODEL(RIG_MICROTUNE, 3)
 
 	/*
 	 * TAPR
@@ -406,6 +419,15 @@
 #define RIG_KIT 25
 #define RIG_BACKEND_KIT "kit"
 #define RIG_MODEL_ELEKTOR304 RIG_MAKE_MODEL(RIG_KIT, 1)
+#define RIG_MODEL_DRT1 RIG_MAKE_MODEL(RIG_KIT, 2)
+
+	/*
+	 * SW/FM/TV tuner cards supported by Video4Linux,*BSD, ..
+	 */
+#define RIG_TUNER 26
+#define RIG_BACKEND_TUNER "tuner"
+#define RIG_MODEL_V4L RIG_MAKE_MODEL(RIG_TUNER, 1)
+
 
 
 	/*
@@ -436,13 +458,11 @@ typedef int rig_model_t;
 		{ RIG_PCR, RIG_BACKEND_PCR }, \
 		{ RIG_AOR, RIG_BACKEND_AOR }, \
 		{ RIG_JRC, RIG_BACKEND_JRC }, \
-		{ RIG_RADIOSHACK, RIG_BACKEND_RADIOSHACK }, \
 		{ RIG_UNIDEN, RIG_BACKEND_UNIDEN }, \
 		{ RIG_DRAKE, RIG_BACKEND_DRAKE }, \
 		{ RIG_LOWE, RIG_BACKEND_LOWE }, \
 		{ RIG_RACAL, RIG_BACKEND_RACAL }, \
 		{ RIG_WJ, RIG_BACKEND_WJ }, \
-		{ RIG_EK, RIG_BACKEND_EK }, \
 		{ RIG_SKANTI, RIG_BACKEND_SKANTI }, \
 		{ RIG_WINRADIO, RIG_BACKEND_WINRADIO }, \
 		{ RIG_TENTEC, RIG_BACKEND_TENTEC }, \
@@ -455,6 +475,7 @@ typedef int rig_model_t;
 		{ RIG_FLEXRADIO, RIG_BACKEND_FLEXRADIO }, \
 		{ RIG_RFT, RIG_BACKEND_RFT }, \
 		{ RIG_KIT, RIG_BACKEND_KIT }, \
+		{ RIG_TUNER, RIG_BACKEND_TUNER }, \
 		{ 0, NULL }, /* end */  \
 }
 
@@ -464,6 +485,9 @@ typedef int rig_model_t;
  *		const char *backend;
  * } rig_backend_list[] = RIG_LIST;
  *
+ * TODO:
+ *
+	{ RIG_RADIOSHACK, RIG_BACKEND_RADIOSHACK }, \
  */
 
 #endif /* _RIGLIST_H */
