@@ -1,8 +1,8 @@
 /*
  *  Hamlib JRC backend - main file
- *  Copyright (c) 2001-2004 by Stephane Fillod
+ *  Copyright (c) 2001-2005 by Stephane Fillod
  *
- *	$Id: jrc.c,v 1.12 2004/05/19 08:57:45 fillods Exp $
+ *	$Id: jrc.c,v 1.21 2005/01/25 00:19:41 fillods Exp $
  *
  *   This library is free software; you can redistribute it and/or modify
  *   it under the terms of the GNU Library General Public License as
@@ -56,8 +56,9 @@
 #define MD_AM	'4'
 #define MD_FM 	'5'
 #define MD_AMS	'6'
-#define MD_ECCS_USB	'7'
-#define MD_ECCS_LSB	'8'
+#define MD_FAX	'6'
+#define MD_ECSS_USB	'7'
+#define MD_ECSS_LSB	'8'
 #define MD_WFM	'9'
 
 
@@ -124,7 +125,7 @@ int jrc_set_freq(RIG *rig, vfo_t vfo, freq_t freq)
 		if (freq >= (freq_t)pow(10, priv->max_freq_len))
 		  return -RIG_EINVAL;
 
-		freq_len = sprintf(freqbuf, "F%0*Ld" EOM, priv->max_freq_len, (long long)freq);
+		freq_len = sprintf(freqbuf, "F%0*"PRIll EOM, priv->max_freq_len, (long long)freq);
 
 		return jrc_transaction (rig, freqbuf, freq_len, NULL, NULL);
 }
@@ -138,13 +139,12 @@ int jrc_get_freq(RIG *rig, vfo_t vfo, freq_t *freq)
 		struct jrc_priv_caps *priv = (struct jrc_priv_caps*)rig->caps->priv;
 		int freq_len, retval;
 		char freqbuf[BUFSZ];
-		long long f;
 
 		//note: JRCs use "I" to get information
 		retval = jrc_transaction (rig, "I" EOM, 2, freqbuf, &freq_len);
 		if (retval != RIG_OK)
 		  return retval;
-				
+
 		//I command returns Iabdffffffffg<CR>
 		if (freqbuf[0] != 'I' || freq_len != priv->info_len) {
 		  rig_debug(RIG_DEBUG_ERR,"jrc_get_freq: wrong answer %s, "
@@ -154,12 +154,38 @@ int jrc_get_freq(RIG *rig, vfo_t vfo, freq_t *freq)
 		freqbuf[4+priv->max_freq_len] = '\0';
 
 		/* extract freq */
-		sscanf(freqbuf+4, "%llu", &f);
-		*freq = f;
+		sscanf(freqbuf+4, "%"SCNfreq, freq);
 
 		return RIG_OK;
 }
 
+/*
+ * jrc_set_vfo
+ * Assumes rig!=NULL
+ */
+int jrc_set_vfo(RIG *rig, vfo_t vfo)
+{
+	unsigned char cmdbuf[16];
+	int cmd_len, retval;
+	char vfo_function;
+
+	switch (vfo) {
+	case RIG_VFO_VFO: vfo_function = 'F'; break;
+	case RIG_VFO_MEM: vfo_function = 'C'; break;
+	default:
+		rig_debug(RIG_DEBUG_ERR,"jrc_set_vfo: unsupported VFO %d\n",
+							vfo);
+		return -RIG_EINVAL;
+	}
+
+	cmd_len = sprintf(cmdbuf, "%c" EOM, vfo_function);
+	
+	retval = jrc_transaction (rig, cmdbuf, cmd_len, NULL, NULL);
+
+	return retval;
+}
+
+
 /*
  * jrc_set_mode
  * Assumes rig!=NULL
@@ -172,7 +198,6 @@ int jrc_set_mode(RIG *rig, vfo_t vfo, rmode_t mode, pbwidth_t width)
 		const char *bandwidth;
 
 		switch (mode) {
-		  /* FIXME: ECSS modes */
 		case RIG_MODE_CW:       amode = MD_CW; break;
 		case RIG_MODE_USB:      amode = MD_USB; break;
 		case RIG_MODE_LSB:      amode = MD_LSB; break;
@@ -181,6 +206,9 @@ int jrc_set_mode(RIG *rig, vfo_t vfo, rmode_t mode, pbwidth_t width)
 		case RIG_MODE_RTTY:	amode = MD_RTTY; break;
 		case RIG_MODE_WFM:	amode = MD_WFM; break;
 		case RIG_MODE_AMS:	amode = MD_AMS; break;
+		case RIG_MODE_FAX:	amode = MD_FAX; break;
+		case RIG_MODE_ECSSUSB:	amode = MD_ECSS_USB; break;
+		case RIG_MODE_ECSSLSB:	amode = MD_ECSS_LSB; break;
 		default:
 		  rig_debug(RIG_DEBUG_ERR,
 			    "jrc_set_mode: unsupported mode %d\n", mode);
@@ -222,7 +250,7 @@ int jrc_get_mode(RIG *rig, vfo_t vfo, rmode_t *mode, pbwidth_t *width)
 		char freqbuf[BUFSZ];
 		char cmode;
 		char cwidth;
-		
+
 		//note: JRCs use "I" to get information
 		retval = jrc_transaction (rig, "I" EOM, 2, freqbuf, &freq_len);
 		if (retval != RIG_OK)
@@ -260,9 +288,14 @@ int jrc_get_mode(RIG *rig, vfo_t vfo, rmode_t *mode, pbwidth_t *width)
 		case '3' : *mode = RIG_MODE_LSB; break;
 		case '4' : *mode = RIG_MODE_AM; break;
 		case '5' : *mode = RIG_MODE_FM; break;
-		case '6' : *mode = RIG_MODE_AMS; break;  //FAX on nrd535
-		case '7' : *mode = RIG_MODE_AMS; break;  //ECSS-USB
-		case '8' : *mode = RIG_MODE_AMS; break;  //ECSS-LSB
+		case '6' : if (rig->caps->rig_model == RIG_MODEL_NRD535) {
+		             *mode = RIG_MODE_FAX; break;  //FAX on nrd535
+					}
+		           else {
+		             *mode = RIG_MODE_AMS; break;  //AMS on nrd545
+				   }
+		case '7' : *mode = RIG_MODE_ECSSUSB; break;  //ECSS-USB
+		case '8' : *mode = RIG_MODE_ECSSLSB; break;  //ECSS-LSB
 		case '9' : *mode = RIG_MODE_WFM; break;  //nrd545 only
 		default:
 		  rig_debug(RIG_DEBUG_ERR,
@@ -356,7 +389,7 @@ int jrc_get_func(RIG *rig, vfo_t vfo, setting_t func, int *status)
 		  }
 		  //*status = funcbuf[1] != '2';
 		  *status = funcbuf[4+priv->max_freq_len] != '2';
-		  
+
 		  return RIG_OK;
 
 		case RIG_FUNC_NB:
@@ -458,19 +491,37 @@ int jrc_set_level(RIG *rig, vfo_t vfo, setting_t level, value_t val)
 		  return jrc_transaction (rig, cmdbuf, cmd_len, NULL, NULL);
 
 		case RIG_LEVEL_SQL:
-		  cmd_len = sprintf(cmdbuf, "JJ%03d" EOM, (int)(val.f*255.0));
+		  cmd_len = sprintf(cmdbuf, "LL%03d" EOM, (int)(val.f*255.0));
 
 		  return jrc_transaction (rig, cmdbuf, cmd_len, NULL, NULL);
 
+		case RIG_LEVEL_NR:
+		  cmd_len = sprintf(cmdbuf, "FF%03d" EOM, (int)(val.f*255.0));
+
+		  return jrc_transaction (rig, cmdbuf, cmd_len, NULL, NULL);
+
+		/*case RIG_LEVEL_TONE:
+		  cmd_len = sprintf(cmdbuf, "KK%03d" EOM, (int)(val.f*255.0));
+
+		  return jrc_transaction (rig, cmdbuf, cmd_len, NULL, NULL);*/
+
 		case RIG_LEVEL_NOTCHF:
-		  cmd_len = sprintf(cmdbuf, "FF%+04d" EOM, val.i);
+		  cmd_len = sprintf(cmdbuf, "GG%+04d" EOM, val.i);
 
 		  return jrc_transaction (rig, cmdbuf, cmd_len, NULL, NULL);
 
+		  /*case RIG_LEVEL_BWC:
+		  if (priv->pbs_len == 3)
+			  val.i /= 10;
+		  
+		  cmd_len = sprintf(cmdbuf, "W%0*d" EOM, priv->pbs_len,  val.i);
+
+		  return jrc_transaction (rig, cmdbuf, cmd_len, NULL, NULL);*/
+
 		case RIG_LEVEL_AGC:
 		  if (val.i < 10)
-		  	cmd_len = sprintf(cmdbuf, "G%d" EOM, 
-				  val.i == RIG_AGC_SLOW ? 0 : 
+		  	cmd_len = sprintf(cmdbuf, "G%d" EOM,
+				  val.i == RIG_AGC_SLOW ? 0 :
 				  val.i == RIG_AGC_FAST ? 1 : 2);
 		  else
 		  	cmd_len = sprintf(cmdbuf, "G3%03d" EOM, val.i/20);
@@ -520,28 +571,39 @@ int jrc_get_level(RIG *rig, vfo_t vfo, setting_t level, value_t *val)
 		  retval = jrc_transaction (rig, "M" EOM, 2, lvlbuf, &lvl_len);
 		  if (retval != RIG_OK)
 		    return retval;
-		  
+
 		  if (lvl_len != 5) {
 		    rig_debug(RIG_DEBUG_ERR,"jrc_get_level: wrong answer"
 			      "len=%d\n", lvl_len);
 		    return -RIG_ERJCTED;
 		  }
-		  
+
 		  lvlbuf[4] = '\0';
 		  val->i = atoi(lvlbuf+1);
 		  break;
-		  
-		case RIG_LEVEL_SQLSTAT:
-		  return -RIG_ENIMPL;	/* get_dcd ? */
+
+		case RIG_LEVEL_STRENGTH:
+		  /* read calibrated A/D converted value */
+		  retval = jrc_transaction (rig, "M" EOM, 2, lvlbuf, &lvl_len);
+		  if (retval != RIG_OK)
+		    return retval;
+
+		  if (lvl_len != 5) {
+		    rig_debug(RIG_DEBUG_ERR,"jrc_get_level: wrong answer"
+			      "len=%d\n", lvl_len);
+		    return -RIG_ERJCTED;
+		  }
+
+		  lvlbuf[4] = '\0';
+		  val->i = (int)rig_raw2val(atoi(lvlbuf+1), &rig->caps->str_cal);
+		  break;
 
 		case RIG_LEVEL_ATT:
-		  //retval = jrc_transaction (rig, "A" EOM, 2, lvlbuf, &lvl_len);
 		  retval = jrc_transaction (rig, "I" EOM, 2, lvlbuf, &lvl_len);
 		  if (retval != RIG_OK)
 		    return retval;
 
-		  //if (lvl_len != 3) {
-		if (lvlbuf[0] != 'I' || lvl_len != priv->info_len) {
+		  if (lvlbuf[0] != 'I' || lvl_len != priv->info_len) {
 		    rig_debug(RIG_DEBUG_ERR,"jrc_get_level: wrong answer"
 			      "len=%d\n", lvl_len);
 		    return -RIG_ERJCTED;
@@ -550,11 +612,37 @@ int jrc_get_level(RIG *rig, vfo_t vfo, setting_t level, value_t *val)
 		  val->i = lvlbuf[1] == '1' ? 20 : 0;
 		  break;
 
+		case RIG_LEVEL_AGC:
+		  retval = jrc_transaction (rig, "I" EOM, 2, lvlbuf, &lvl_len);
+		  if (retval != RIG_OK)
+		    return retval;
+
+		  if (lvlbuf[0] != 'I' || lvl_len != priv->info_len) {
+		    rig_debug(RIG_DEBUG_ERR,"jrc_get_level: wrong answer"
+			      "len=%d\n", lvl_len);
+		    return -RIG_ERJCTED;
+		  }
+
+		  lvlbuf[priv->info_len-1] = '\0';
+
+		  if (priv->info_len==14) {
+		    switch (lvlbuf[priv->info_len-2]){
+		    case '0' : val->i = RIG_AGC_SLOW; break;
+		    case '1' : val->i = RIG_AGC_FAST; break;
+		    case '2' : val->i = RIG_AGC_OFF; break;
+		    default : val->i = RIG_AGC_FAST;
+		    }
+		  }
+		  else {
+		    val->i = atoi(lvlbuf+priv->info_len-4);
+		  }
+		  break;
+
 		case RIG_LEVEL_RF:
 		  retval = jrc_transaction (rig, "HH" EOM, 3, lvlbuf, &lvl_len);
 		  if (retval != RIG_OK)
 		    return retval;
-		  
+
 		  if (lvl_len != 6) {
 		    rig_debug(RIG_DEBUG_ERR,"jrc_get_level: wrong answer"
 			      "len=%d\n", lvl_len);
@@ -571,7 +659,7 @@ int jrc_get_level(RIG *rig, vfo_t vfo, setting_t level, value_t *val)
 		  retval = jrc_transaction (rig, "JJ" EOM, 3, lvlbuf, &lvl_len);
 		  if (retval != RIG_OK)
 		    return retval;
-		  
+
 		  if (lvl_len != 6) {
 		    rig_debug(RIG_DEBUG_ERR,"jrc_get_level: wrong answer"
 			      "len=%d\n", lvl_len);
@@ -583,12 +671,12 @@ int jrc_get_level(RIG *rig, vfo_t vfo, setting_t level, value_t *val)
 		  sscanf(lvlbuf+2, "%u", &lvl);
 		  val->f = (float)lvl/255.0;
 		  break;
-		  
+
 		case RIG_LEVEL_SQL:
 		  retval = jrc_transaction (rig, "LL" EOM, 3, lvlbuf, &lvl_len);
 		  if (retval != RIG_OK)
 		    return retval;
-		  
+
 		  if (lvl_len != 6) {
 		    rig_debug(RIG_DEBUG_ERR,"jrc_get_level: wrong answer"
 			      "len=%d\n", lvl_len);
@@ -600,12 +688,44 @@ int jrc_get_level(RIG *rig, vfo_t vfo, setting_t level, value_t *val)
 		  sscanf(lvlbuf+2, "%u", &lvl);
 		  val->f = (float)lvl/255.0;
 		  break;
-		  
+
+		case RIG_LEVEL_NR:
+		  retval = jrc_transaction (rig, "FF" EOM, 3, lvlbuf, &lvl_len);
+		  if (retval != RIG_OK)
+		    return retval;
+
+		  if (lvl_len != 6) {
+		    rig_debug(RIG_DEBUG_ERR,"jrc_get_level: wrong answer"
+			      "len=%d\n", lvl_len);
+		    return -RIG_ERJCTED;
+		  }
+		  /*
+		   * 000..255
+		   */
+		  sscanf(lvlbuf+2, "%u", &lvl);
+		  val->f = (float)lvl/255.0;
+		  break;
+
+		/*case RIG_LEVEL_TONE:
+		  retval = jrc_transaction (rig, "KK" EOM, 3, lvlbuf, &lvl_len);
+		  if (retval != RIG_OK)
+		    return retval;
+
+		  if (lvl_len != 6) {
+		    rig_debug(RIG_DEBUG_ERR,"jrc_get_level: wrong answer"
+			      "len=%d\n", lvl_len);
+		    return -RIG_ERJCTED;
+		  }
+
+		  sscanf(lvlbuf+2, "%u", &lvl);
+		  val->f = (float)lvl/255.0;
+		  break;*/
+
 		case RIG_LEVEL_NOTCHF:
 		  retval = jrc_transaction (rig, "GG" EOM, 3, lvlbuf, &lvl_len);
 		  if (retval != RIG_OK)
 		    return retval;
-		  
+
 		  if (lvl_len != 8) {
 		    rig_debug(RIG_DEBUG_ERR,"jrc_get_level: wrong answer"
 			      "len=%d\n", lvl_len);
@@ -618,13 +738,31 @@ int jrc_get_level(RIG *rig, vfo_t vfo, setting_t level, value_t *val)
 		  val->f = (float)lvl/255.0;
 		  break;
 
+		  /*case RIG_LEVEL_BWC:
+		  retval = jrc_transaction (rig, "W" EOM, 2, lvlbuf, &lvl_len);
+		  if (retval != RIG_OK)
+		    return retval;
+
+		  if (lvlbuf[0] != 'W' || lvl_len != priv->pbs_len+2) {
+		    rig_debug(RIG_DEBUG_ERR,"jrc_get_level: wrong answer"
+			      "len=%d\n", lvl_len);
+		    return -RIG_ERJCTED;
+		  }
+
+		  sscanf(lvlbuf+1, "%d", &lvl);
+		  if (priv->pbs_len == 3)
+			  lvl *= 10;
+		  
+		  val->i = lvl;
+		  break;*/
+
 		case RIG_LEVEL_CWPITCH:
 		  cw_len = sprintf(cwbuf, "%s" EOM, priv->cw_pitch);
 
 		  retval = jrc_transaction (rig, cwbuf, cw_len, lvlbuf, &lvl_len);
 		  if (retval != RIG_OK)
 		    return retval;
-		  
+
 		  if (lvl_len != cw_len+5) {
 		    rig_debug(RIG_DEBUG_ERR,"jrc_get_level: wrong answer"
 			      "len=%d\n", lvl_len);
@@ -639,8 +777,8 @@ int jrc_get_level(RIG *rig, vfo_t vfo, setting_t level, value_t *val)
 		  retval = jrc_transaction (rig, "P" EOM, 2, lvlbuf, &lvl_len);
 		  if (retval != RIG_OK)
 		    return retval;
-		  
-		if (lvlbuf[0] != 'P' || lvl_len != priv->pbs_info_len) {
+
+		  if (lvlbuf[0] != 'P' || lvl_len != priv->pbs_info_len) {
 		    rig_debug(RIG_DEBUG_ERR,"jrc_get_level: wrong answer"
 			      "len=%d\n", lvl_len);
 		    return -RIG_ERJCTED;
@@ -684,23 +822,22 @@ int jrc_set_parm(RIG *rig, setting_t parm, value_t val)
 
 		case RIG_PARM_BEEP:
 
-		  //cmd_len = sprintf(cmdbuf, "U%03d" EOM, val.i?101:100);
 		  cmd_len = sprintf(cmdbuf, "U%0*d" EOM, priv->beep_len, priv->beep + val.i?1:0);
-		  
+
 		  return jrc_transaction (rig, cmdbuf, cmd_len, NULL, NULL);
 
 		case RIG_PARM_TIME:
 		  minutes = val.i/60;
-		  cmd_len = sprintf(cmdbuf, "R1%02d%02d" EOM, 
+		  cmd_len = sprintf(cmdbuf, "R1%02d%02d" EOM,
 				    minutes/60, minutes%60);
-		  
+
 		  return jrc_transaction (rig, cmdbuf, cmd_len, NULL, NULL);
-		  
+
 		default:
 		  rig_debug(RIG_DEBUG_ERR,"Unsupported set_parm %d\n", parm);
 		  return -RIG_EINVAL;
 		}
-		
+
 		return RIG_OK;
 }
 
@@ -710,8 +847,11 @@ int jrc_set_parm(RIG *rig, setting_t parm, value_t val)
  */
 int jrc_get_parm(RIG *rig, setting_t parm, value_t *val)
 {
+		struct jrc_priv_caps *priv = (struct jrc_priv_caps*)rig->caps->priv;
                 int retval, lvl_len, i;
 		char lvlbuf[BUFSZ];
+		char cmdbuf[BUFSZ];
+		int cmd_len;
 
 		/* Optimize:
 		 *   sort the switch cases with the most frequent first
@@ -721,14 +861,14 @@ int jrc_get_parm(RIG *rig, setting_t parm, value_t *val)
 		  retval = jrc_transaction (rig, "R0" EOM, 3, lvlbuf, &lvl_len);
 		  if (retval != RIG_OK)
 		    return retval;
-		  
+
 		  /* "Rhhmmss"CR */
 		  if (lvl_len != 8) {
 		    rig_debug(RIG_DEBUG_ERR,"jrc_get_parm: wrong answer"
 			      "len=%d\n", lvl_len);
 		    return -RIG_ERJCTED;
 		  }
-		  
+
 		  /* convert ASCII to numeric 0..9 */
 		  for (i=1; i<7; i++) {
 		    lvlbuf[i] -= '0';
@@ -737,21 +877,22 @@ int jrc_get_parm(RIG *rig, setting_t parm, value_t *val)
 			     10*lvlbuf[3] + lvlbuf[4])*60 +	/* minutes */
 		             10*lvlbuf[5] + lvlbuf[6];	/* secondes */
 		  break;
-		  
+
 		case RIG_PARM_BEEP:
-		  retval = jrc_transaction (rig, "U9" EOM, 3, lvlbuf, &lvl_len);
+		  cmd_len = sprintf(cmdbuf, "U%d" EOM, priv->beep/10);
+		  retval = jrc_transaction (rig, cmdbuf, cmd_len, lvlbuf, &lvl_len);
 		  if (retval != RIG_OK)
 		    return retval;
-		  
-		  if (lvl_len != 4) {
+
+		  if (lvl_len != priv->beep_len+2) {
 		    rig_debug(RIG_DEBUG_ERR,"jrc_get_parm: wrong answer"
 			      "len=%d\n", lvl_len);
 		    return -RIG_ERJCTED;
 		  }
 
-		  val->i = lvlbuf[2] == 0 ? 0 : 1;
+		  val->i = lvlbuf[priv->beep_len] == 0 ? 0 : 1;
 		  break;
-		  
+
 		default:
 		  rig_debug(RIG_DEBUG_ERR,"Unsupported get_parm %d\n", parm);
 		  return -RIG_EINVAL;
@@ -811,6 +952,30 @@ int jrc_set_powerstat(RIG *rig, powerstat_t status)
 		return jrc_transaction (rig, pwrbuf, pwr_len, NULL, NULL);
 }
 
+/*
+ * jrc_get_powerstat
+ * Assumes rig!=NULL
+ */
+int jrc_get_powerstat(RIG *rig, powerstat_t *status)
+//powerstat_t jrc_get_powerstat(RIG *rig)
+{
+		char pwrbuf[BUFSZ];
+		int pwr_len, retval;
+
+		retval = jrc_transaction (rig, "T" EOM, 2, pwrbuf, &pwr_len);
+		if (retval != RIG_OK)
+		  return retval;
+
+		if (pwr_len != 3) {
+		  rig_debug(RIG_DEBUG_ERR,"jrc_get_powerstat: wrong answer %s, "
+			    "len=%d\n", pwrbuf, pwr_len);
+		  return -RIG_ERJCTED;
+		}
+		*status = pwrbuf[1] == '0' ? RIG_POWER_OFF : RIG_POWER_ON;
+
+		return RIG_OK;
+}
+
 /*
  * jrc_reset
  * Assumes rig!=NULL
@@ -825,7 +990,7 @@ int jrc_reset(RIG *rig, reset_t reset)
 			case RIG_RESET_MCALL: rst='1'; break;	/* mem clear */
 			case RIG_RESET_VFO: rst='2'; break;		/* user setup default */
 			case RIG_RESET_MASTER: rst='3'; break;	/* 1 + 2 */
-			default: 
+			default:
 				rig_debug(RIG_DEBUG_ERR,"jrc_reset: unsupported reset %d\n",
 								reset);
 				return -RIG_EINVAL;
@@ -870,7 +1035,8 @@ int jrc_get_mem(RIG *rig, vfo_t vfo, int *ch)
 		if (retval != RIG_OK)
 		  return retval;
 
-		if (mem_len != priv->mem_len) {
+		/* need to handle vacant memories LmmmV<cr>, len = 6 */
+		if ((mem_len != priv->mem_len) && (mem_len != 6)) {
 		  rig_debug(RIG_DEBUG_ERR,"jrc_get_mem: wrong answer %s, "
 			    "len=%d\n", membuf, mem_len);
 		  return -RIG_ERJCTED;
@@ -884,6 +1050,163 @@ int jrc_get_mem(RIG *rig, vfo_t vfo, int *ch)
 		return RIG_OK;
 }
 
+/*
+ * jrc_set_chan
+ * Assumes rig!=NULL
+ */
+int jrc_set_chan(RIG *rig, const channel_t *chan)
+{
+	struct jrc_priv_caps *priv = (struct jrc_priv_caps*)rig->caps->priv;
+	char	cmdbuf[BUFSZ];
+	int		cmd_len;
+
+	sprintf(cmdbuf,"K%03d000",chan->channel_num);
+	
+	if (chan->levels[rig_setting2idx(RIG_LEVEL_ATT)].i == 20)
+		cmdbuf[4] = '1';
+			
+	if (chan->width <= s_Hz(1500))
+		cmdbuf[5] = '2';
+	else if (chan->width <= s_Hz(4000))
+		cmdbuf[5] = '1';
+	else if (chan->width <= s_Hz(9000))
+		cmdbuf[5] = '0';
+	else if (rig->caps->rig_model == RIG_MODEL_NRD535)
+		cmdbuf[5] = '3'; /*aux - nrd535 only*/
+	else
+		cmdbuf[5] = '1'; /*inter*/
+	
+	switch (chan->mode) {
+		case RIG_MODE_CW:       cmdbuf[6] = MD_CW; break;
+		case RIG_MODE_USB:      cmdbuf[6] = MD_USB; break;
+		case RIG_MODE_LSB:      cmdbuf[6] = MD_LSB; break;
+		case RIG_MODE_FM:       cmdbuf[6] = MD_FM; break;
+		case RIG_MODE_AM:       cmdbuf[6] = MD_AM; break;
+		case RIG_MODE_RTTY:		cmdbuf[6] = MD_RTTY; break;
+		case RIG_MODE_WFM:		cmdbuf[6] = MD_WFM; break;
+		case RIG_MODE_AMS:		cmdbuf[6] = MD_AMS; break;
+		case RIG_MODE_FAX:		cmdbuf[6] = MD_FAX; break;
+		case RIG_MODE_ECSSUSB:	cmdbuf[6] = MD_ECSS_USB; break;
+		case RIG_MODE_ECSSLSB:	cmdbuf[6] = MD_ECSS_LSB; break;
+		default: cmdbuf[6] = MD_AM;
+	}
+	
+	sprintf(cmdbuf+7,"%0*"PRIll, priv->max_freq_len, (long long)chan->freq);
+	
+	if (priv->mem_len==17) {
+		switch (chan->levels[rig_setting2idx(RIG_LEVEL_AGC)].i) {
+			case RIG_AGC_SLOW : cmdbuf[priv->mem_len-2] = '0'; break;
+			case RIG_AGC_FAST : cmdbuf[priv->mem_len-2] = '1'; break;
+			case RIG_AGC_OFF  : cmdbuf[priv->mem_len-2] = '2'; break;
+			default : cmdbuf[priv->mem_len-2] = '1';
+		}
+	}
+	else {
+		sprintf(cmdbuf+priv->mem_len-4,"%03d", chan->levels[rig_setting2idx(RIG_LEVEL_AGC)].i);
+	}
+
+	cmd_len = priv->mem_len;	
+	return jrc_transaction (rig, cmdbuf, cmd_len, NULL, NULL);
+}
+
+/*
+ * jrc_get_chan
+ * Assumes rig!=NULL
+ */
+int jrc_get_chan(RIG *rig, channel_t *chan)
+{
+	struct jrc_priv_caps *priv = (struct jrc_priv_caps*)rig->caps->priv;
+	char	membuf[BUFSZ], cmdbuf[BUFSZ], freqbuf[BUFSZ];
+	int		mem_len, cmd_len, retval;
+
+	chan->vfo = RIG_VFO_MEM;
+	chan->ant = RIG_ANT_NONE;
+	chan->freq = 0;
+	chan->mode = RIG_MODE_NONE;
+	chan->width = RIG_PASSBAND_NORMAL;
+	chan->tx_freq = 0;
+	chan->tx_mode = RIG_MODE_NONE;
+	chan->tx_width = RIG_PASSBAND_NORMAL;
+	chan->split = RIG_SPLIT_OFF;
+	chan->tx_vfo = RIG_VFO_NONE;
+	chan->rptr_shift = RIG_RPT_SHIFT_NONE;
+	chan->rptr_offs = 0;
+	chan->tuning_step = 0;
+	chan->rit = 0;
+	chan->xit = 0;
+	chan->funcs = 0;
+	chan->levels[rig_setting2idx(RIG_LEVEL_AGC)].i = RIG_AGC_OFF;
+	chan->levels[rig_setting2idx(RIG_LEVEL_ATT)].i = 0;
+	chan->ctcss_tone = 0;
+	chan->ctcss_sql = 0;
+	chan->dcs_code = 0;
+	chan->dcs_sql = 0;
+	chan->scan_group = 0;
+	chan->flags = RIG_CHFLAG_SKIP;
+	strcpy(chan->channel_desc, "");
+
+	cmd_len = sprintf(cmdbuf, "L%03d%03d" EOM, chan->channel_num, chan->channel_num);
+	retval = jrc_transaction (rig, cmdbuf, cmd_len, membuf, &mem_len);
+	if (retval != RIG_OK)
+		return retval;
+
+	/* need to handle vacant memories LmmmV<cr>, len = 6 */
+	if ((mem_len != priv->mem_len) && (mem_len != 6)) {
+		rig_debug(RIG_DEBUG_ERR,"jrc_get_mem: wrong answer %s, "
+				    "len=%d\n", membuf, mem_len);
+		return -RIG_ERJCTED;
+	}
+
+	if (mem_len != 6) {
+		if (membuf[4] == '1')
+			chan->levels[rig_setting2idx(RIG_LEVEL_ATT)].i = 20;
+		switch (membuf[5]) {
+			case '0' : chan->width = s_Hz(6000); break; //wide
+			case '1' : chan->width = s_Hz(2000); break; //inter
+			case '2' : chan->width = s_Hz(1000); break; //narr
+			case '3' : chan->width = s_Hz(12000); break; //aux - nrd535 only
+			default : chan->width = RIG_PASSBAND_NORMAL;
+		}
+		switch (membuf[6]) {
+			case '0' : chan->mode = RIG_MODE_RTTY; break;
+			case '1' : chan->mode = RIG_MODE_CW; break;
+			case '2' : chan->mode = RIG_MODE_USB; break;
+			case '3' : chan->mode = RIG_MODE_LSB; break;
+			case '4' : chan->mode = RIG_MODE_AM; break;
+			case '5' : chan->mode = RIG_MODE_FM; break;
+			case '6' : if (rig->caps->rig_model == RIG_MODEL_NRD535) {
+		             		chan->mode = RIG_MODE_FAX; break;  //FAX on nrd535
+						}
+						else {
+							chan->mode = RIG_MODE_AMS; break;  //AMS on nrd545
+						}
+			case '7' : chan->mode = RIG_MODE_ECSSUSB; break;  //ECSS-USB
+			case '8' : chan->mode = RIG_MODE_ECSSLSB; break;  //ECSS-LSB
+			case '9' : chan->mode = RIG_MODE_WFM; break;  //nrd545 only
+			default: chan->mode = RIG_MODE_NONE;
+		}
+
+		strncpy(freqbuf,membuf+7,priv->max_freq_len);
+		freqbuf[priv->max_freq_len] = 0x00;
+		chan->freq = strtol(freqbuf,NULL, 10);
+		
+		if (priv->mem_len==17) {
+			switch (membuf[priv->mem_len-2]){
+				case '0' : chan->levels[rig_setting2idx(RIG_LEVEL_AGC)].i = RIG_AGC_SLOW; break;
+				case '1' : chan->levels[rig_setting2idx(RIG_LEVEL_AGC)].i = RIG_AGC_FAST; break;
+				case '2' : chan->levels[rig_setting2idx(RIG_LEVEL_AGC)].i = RIG_AGC_OFF; break;
+				default : chan->levels[rig_setting2idx(RIG_LEVEL_AGC)].i = RIG_AGC_FAST;
+		    }
+		}
+		else {
+			strncpy(freqbuf,membuf+priv->mem_len-4,3);
+			chan->levels[rig_setting2idx(RIG_LEVEL_AGC)].i = strtol(freqbuf,NULL, 10);
+		}
+	}
+
+	return RIG_OK;
+}
+
 /*
  * jrc_vfo_op
  * Assumes rig!=NULL
@@ -894,7 +1217,7 @@ int jrc_vfo_op(RIG *rig, vfo_t vfo, vfo_op_t op)
 
 		switch(op) {
 			case RIG_OP_FROM_VFO: cmd="E1" EOM; break;
-			default: 
+			default:
 				rig_debug(RIG_DEBUG_ERR,"jrc_vfo_op: unsupported op %#x\n",
 								op);
 				return -RIG_EINVAL;
@@ -908,7 +1231,7 @@ int jrc_vfo_op(RIG *rig, vfo_t vfo, vfo_op_t op)
  * jrc_scan, scan operation
  * Assumes rig!=NULL
  *
- * Not really a scan operation so speaking. 
+ * Not really a scan operation so speaking.
  * You just make the rig increment frequency of decrement continuously,
  * depending on the sign of ch.
  * However, using DCD sensing, followed by a stop, you get it.
@@ -932,17 +1255,25 @@ int jrc_scan(RIG *rig, vfo_t vfo, scan_t scan, int ch)
 		return jrc_transaction (rig, scan_cmd, 3, NULL, NULL);
 }
 
-static int jrc2rig_mode(RIG *rig, char jmode, char jwidth, 
+static int jrc2rig_mode(RIG *rig, char jmode, char jwidth,
 				rmode_t *mode, pbwidth_t *width)
 {
 		switch (jmode) {
-		case MD_CW:	*mode = RIG_MODE_CW; break;
-		case MD_USB:	*mode = RIG_MODE_USB; break;
-		case MD_LSB:	*mode = RIG_MODE_LSB; break;
-		case MD_FM:	*mode = RIG_MODE_FM; break;
-		case MD_AM:	*mode = RIG_MODE_AM; break;
-		case MD_RTTY:	*mode = RIG_MODE_RTTY; break;
-		case MD_WFM:	*mode = RIG_MODE_WFM; break;
+		case MD_CW:			*mode = RIG_MODE_CW; break;
+		case MD_USB:		*mode = RIG_MODE_USB; break;
+		case MD_LSB:		*mode = RIG_MODE_LSB; break;
+		case MD_FM:			*mode = RIG_MODE_FM; break;
+		case MD_AM:			*mode = RIG_MODE_AM; break;
+		case MD_AMS:		if (rig->caps->rig_model == RIG_MODEL_NRD535) {
+								*mode = RIG_MODE_FAX; break;
+							}
+							else {
+								*mode = RIG_MODE_AMS; break;
+							}
+		case MD_ECSS_USB:	*mode = RIG_MODE_ECSSUSB; break;
+		case MD_ECSS_LSB:	*mode = RIG_MODE_ECSSLSB; break;
+		case MD_RTTY:		*mode = RIG_MODE_RTTY; break;
+		case MD_WFM:		*mode = RIG_MODE_WFM; break;
 		default:
 		  rig_debug(RIG_DEBUG_ERR,
 			    "jrc_set_mode: unsupported mode %c\n",
@@ -985,7 +1316,7 @@ int jrc_decode_event(RIG *rig)
 		struct rig_state *rs;
 		freq_t freq;
 		rmode_t mode;
-		pbwidth_t width; 
+		pbwidth_t width;
 		int count;
 		char buf[BUFSZ];
 
@@ -1014,22 +1345,20 @@ int jrc_decode_event(RIG *rig)
 		 */
 
 		if (rig->callbacks.freq_event) {
-		  long long f;
-		  
+
 		  //buf[14] = '\0';	/* side-effect: destroy AGC first digit! */
 		  buf[4+priv->max_freq_len] = '\0';	/* side-effect: destroy AGC first digit! */
-		  sscanf(buf+4, "%lld", &f);
-		  freq = f;
+		  sscanf(buf+4, "%"SCNfreq, &freq);
 		  return rig->callbacks.freq_event(rig, RIG_VFO_CURR, freq,
 						   rig->callbacks.freq_arg);
 		}
-		
+
 		if (rig->callbacks.mode_event) {
 		  jrc2rig_mode(rig, buf[3], buf[2], &mode, &width);
 		  return rig->callbacks.mode_event(rig, RIG_VFO_CURR, mode, width,
 						   rig->callbacks.freq_arg);
 		}
-		
+
 		return RIG_OK;
 }
 
