@@ -1,8 +1,8 @@
 /*
  *  Hamlib Kenwood backend - main file
- *  Copyright (c) 2000-2004 by Stephane Fillod and others
+ *  Copyright (c) 2000-2005 by Stephane Fillod and others
  *
- *	$Id: kenwood.c,v 1.76 2004/05/02 17:17:31 fillods Exp $
+ *	$Id: kenwood.c,v 1.87 2005/03/13 17:54:16 fillods Exp $
  *
  *   This library is free software; you can redistribute it and/or modify
  *   it under the terms of the GNU Library General Public License as
@@ -85,6 +85,7 @@ static const struct kenwood_id kenwood_id_list[] = {
 	{ RIG_MODEL_TS570D, 17 },	/* Elecraft K2 also returns 17 */
 	{ RIG_MODEL_TS570S, 18 },
 	{ RIG_MODEL_TS2000, 19 },
+	{ RIG_MODEL_TS480, 20 },
 	{ RIG_MODEL_NONE, UNKNOWN_ID },	/* end marker */
 };
 
@@ -187,8 +188,8 @@ transaction_read:
 
     /* Command recognised by rig but invalid data entered. */
     if (strlen(data) == 2 && data[0] == 'N') {
-        rig_debug(RIG_DEBUG_ERR, "%s: NegAck for '%s'\n", __FUNCTION__, cmdstr);
-        retval = -RIG_ERJCTED;
+        rig_debug(RIG_DEBUG_VERBOSE, "%s: NegAck for '%s'\n", __FUNCTION__, cmdstr);
+        retval = -RIG_ENAVAIL;
         goto transaction_quit;
     }
 
@@ -305,15 +306,17 @@ int kenwood_set_split_vfo(RIG *rig, vfo_t vfo, split_t split, vfo_t txvfo)
 					rig_debug(RIG_DEBUG_ERR,"kenwood_set_split_vfo: unsupported VFO %d\n", txvfo);
 					return -RIG_EINVAL;
 			} 
-		} else
-		  if(vfo==RIG_VFO_CURR)
-			rig_debug(RIG_DEBUG_ERR,"kenwood_set_split_vfo: unsupported VFO %d\n", vfo);
-
 		/* set TX VFO */
 		cmd_len = sprintf(cmdbuf, "FT%c%s", vfo_function, cmd_trm(rig));
 		ack_len = 0;
 		retval = kenwood_transaction (rig, cmdbuf, cmd_len, ackbuf, &ack_len);
-		return retval;
+		if (retval != RIG_OK)
+			return retval;
+		} else
+		  if(vfo==RIG_VFO_CURR)
+			rig_debug(RIG_DEBUG_ERR,"kenwood_set_split_vfo: unsupported VFO %d\n", vfo);
+
+		return RIG_OK;
 }
 
 /*
@@ -373,7 +376,7 @@ int kenwood_set_freq(RIG *rig, vfo_t vfo, freq_t freq)
 								vfo);
 			return -RIG_EINVAL;
 		}
-		freq_len = sprintf(freqbuf,"F%c%011Ld;", vfo_letter, (long long)freq);
+		freq_len = sprintf(freqbuf,"F%c%011"PRIll";", vfo_letter, (long long)freq);
 
 		ack_len = 0;
 		retval = kenwood_transaction (rig, freqbuf, freq_len, ackbuf, &ack_len);
@@ -391,7 +394,6 @@ int kenwood_get_freq(RIG *rig, vfo_t vfo, freq_t *freq)
 		unsigned char cmdbuf[4];
 		int cmd_len, freq_len, retval;
 		char vfo_letter;
-		long long f;
 		vfo_t	tvfo;
 
         	if(vfo==RIG_VFO_CURR) tvfo=rig->state.current_vfo;
@@ -402,8 +404,8 @@ int kenwood_get_freq(RIG *rig, vfo_t vfo, freq_t *freq)
 		case RIG_VFO_B: vfo_letter = 'B'; break;
 		case RIG_VFO_C: vfo_letter = 'C'; break;
 		default: 
-			rig_debug(RIG_DEBUG_ERR,"kenwood_get_freq: unsupported VFO %d\n",
-								vfo);
+			rig_debug(RIG_DEBUG_ERR,"%s: unsupported VFO %d\n",
+					__FUNCTION__, vfo);
 			return -RIG_EINVAL;
 		}
 
@@ -415,13 +417,12 @@ int kenwood_get_freq(RIG *rig, vfo_t vfo, freq_t *freq)
 			return retval;
 
 		if (freq_len != 14 || freqbuf[0] != 'F') {
-			rig_debug(RIG_DEBUG_ERR,"kenwood_get_freq: unexpected answer %s, "
-							"len=%d\n", freqbuf, freq_len);
+			rig_debug(RIG_DEBUG_ERR,"%s: unexpected answer %s, "
+					"len=%d\n", __FUNCTION__, freqbuf, freq_len);
 			return -RIG_ERJCTED;
 		}
 
-		sscanf(freqbuf+2, "%lld", &f);
-		*freq = (freq_t)f;
+		sscanf(freqbuf+2, "%"SCNfreq, freq);
 
 		return RIG_OK;
 }
@@ -578,7 +579,7 @@ int kenwood_set_level(RIG *rig, vfo_t vfo, setting_t level, value_t val)
 {
 		unsigned char levelbuf[16], ackbuf[16];
 		int level_len, ack_len, retval;
-		int kenwood_val;
+		int i, kenwood_val;
 
 		if (RIG_LEVEL_IS_FLOAT(level))
 				kenwood_val = val.f * 255;
@@ -607,6 +608,18 @@ int kenwood_set_level(RIG *rig, vfo_t vfo, setting_t level, value_t val)
 		level_len = sprintf(levelbuf, "GT%03d;", 84*kenwood_val);
 		break;
 
+		case RIG_LEVEL_ATT:
+		/* set the attenuator if a correct value is entered */
+		for (i=0; i<MAXDBLSTSIZ; i++)
+			if (kenwood_val == rig->state.attenuator[i])
+			{
+				level_len = sprintf(levelbuf, "RA%02d;", kenwood_val/6);
+				break;
+			}
+			else
+				level_len = sprintf(levelbuf, "RA00;");
+		break;
+
 		default:
 			rig_debug(RIG_DEBUG_ERR,"Unsupported set_level %d", level);
 			return -RIG_EINVAL;
@@ -624,7 +637,7 @@ int kenwood_set_level(RIG *rig, vfo_t vfo, setting_t level, value_t val)
 /* 
  * assumes f!=NULL
  */
-static int get_kenwood_level(RIG *rig, const char *cmd, int cmd_len, float *f)
+int get_kenwood_level(RIG *rig, const char *cmd, int cmd_len, float *f)
 {
 	unsigned char lvlbuf[50];
 	int lvl_len, retval;
@@ -730,7 +743,6 @@ int kenwood_get_level(RIG *rig, vfo_t vfo, setting_t level, value_t *val)
 			else if (agclevel <= 255) val->i = 3;
 			return ret;
 
-		case RIG_LEVEL_SQLSTAT:
 		case RIG_LEVEL_PREAMP:
 		case RIG_LEVEL_IF:
 		case RIG_LEVEL_APF:
@@ -765,43 +777,47 @@ int kenwood_set_func(RIG *rig, vfo_t vfo, setting_t func, int status)
 		ack_len = 0;
 		switch (func) {
 		case RIG_FUNC_NB:
-		fct_len = sprintf(fctbuf,"NB%c;", status==RIG_FUNC_NB?'0':'1');
+		fct_len = sprintf(fctbuf,"NB%c;", (0==status)?'0':'1');
 		return kenwood_transaction (rig, fctbuf, fct_len, ackbuf, &ack_len);
 
 		case RIG_FUNC_ABM:
-		fct_len = sprintf(fctbuf,"AM%c;", status==RIG_FUNC_ABM?'0':'1');
+		fct_len = sprintf(fctbuf,"AM%c;", (0==status)?'0':'1');
 		return kenwood_transaction (rig, fctbuf, fct_len, ackbuf, &ack_len);
 
 		case RIG_FUNC_COMP:
-		fct_len = sprintf(fctbuf,"PR%c;", status==RIG_FUNC_COMP?'0':'1');
+		fct_len = sprintf(fctbuf,"PR%c;", (0==status)?'0':'1');
 		return kenwood_transaction (rig, fctbuf, fct_len, ackbuf, &ack_len);
 
 		case RIG_FUNC_TONE:
-		fct_len = sprintf(fctbuf,"TO%c;", status==RIG_FUNC_TONE?'0':'1');
+		fct_len = sprintf(fctbuf,"TO%c;", (0==status)?'0':'1');
 		return kenwood_transaction (rig, fctbuf, fct_len, ackbuf, &ack_len);
 
 		case RIG_FUNC_TSQL:
-		fct_len = sprintf(fctbuf,"CT%c;", status==RIG_FUNC_TSQL?'0':'1');
+		fct_len = sprintf(fctbuf,"CT%c;", (0==status)?'0':'1');
 		return kenwood_transaction (rig, fctbuf, fct_len, ackbuf, &ack_len);
 
 		case RIG_FUNC_VOX:
-		fct_len = sprintf(fctbuf,"VX%c;", status==RIG_FUNC_VOX?'0':'1');
+		fct_len = sprintf(fctbuf,"VX%c;", (0==status)?'0':'1');
+		return kenwood_transaction (rig, fctbuf, fct_len, ackbuf, &ack_len);
+
+		case RIG_FUNC_FAGC:
+		fct_len = sprintf(fctbuf,"GT00%c;", (0==status)? '4':'2');
 		return kenwood_transaction (rig, fctbuf, fct_len, ackbuf, &ack_len);
 
 		case RIG_FUNC_NR:
-		fct_len = sprintf(fctbuf,"NR%c;", status==RIG_FUNC_NR?'0':'1');
+		fct_len = sprintf(fctbuf,"NR%c;", (0==status)?'0':'1');
 		return kenwood_transaction (rig, fctbuf, fct_len, ackbuf, &ack_len);
 
 		case RIG_FUNC_BC:
-		fct_len = sprintf(fctbuf,"BC%c;", status==RIG_FUNC_BC?'0':'1');
+		fct_len = sprintf(fctbuf,"BC%c;", (0==status)?'0':'1');
 		return kenwood_transaction (rig, fctbuf, fct_len, ackbuf, &ack_len);
 
 		case RIG_FUNC_ANF:
-		fct_len = sprintf(fctbuf,"NT%c;", status==RIG_FUNC_ANF?'0':'1');
+		fct_len = sprintf(fctbuf,"NT%c;", (0==status)?'0':'1');
 		return kenwood_transaction (rig, fctbuf, fct_len, ackbuf, &ack_len);
 
 		case RIG_FUNC_LOCK:
-		fct_len = sprintf(fctbuf,"LK%c;", status==RIG_FUNC_LOCK?'0':'1');
+		fct_len = sprintf(fctbuf,"LK%c;", (0==status)?'0':'1');
 		return kenwood_transaction (rig, fctbuf, fct_len, ackbuf, &ack_len);
 
 		default:
@@ -1154,7 +1170,7 @@ int kenwood_reset(RIG *rig, reset_t reset)
  */
 int kenwood_send_morse(RIG *rig, vfo_t vfo, const char *msg)
 {
-		unsigned char morsebuf[30], ackbuf[16];
+		unsigned char morsebuf[30], m2[30], ackbuf[16];
 		int morse_len, ack_len;
 		int msg_len, buff_len, retval;
 		const char *p;
@@ -1166,13 +1182,18 @@ int kenwood_send_morse(RIG *rig, vfo_t vfo, const char *msg)
 				/*
 				 * TODO: check with "KY;" if char buffer is available.
 				 * 		if not, sleep.
+				 *
+				 * Make the total message segments 28 characters 
+				 * in length because Kenwood demands it.
+				 * Spaces fill in the message end.
 				 */
 				buff_len = msg_len > 24 ? 24 : msg_len;
 
-				strcpy(morsebuf, "KY ");
-				strncat(morsebuf, p, buff_len);
-				strcat(morsebuf, ";");
-				morse_len = 4 + buff_len;
+				strncpy(m2, p, 24);
+				m2[24] = '\0';
+
+				/* morse_len must be 28 */
+				morse_len = sprintf(morsebuf,"KY %-24s;", m2);
 				ack_len = 0;
 				retval = kenwood_transaction (rig, morsebuf, morse_len, 
 								ackbuf, &ack_len);
@@ -1303,7 +1324,7 @@ DECLARE_PROBERIG_BACKEND(kenwood)
 	unsigned char idbuf[IDBUFSZ];
 	int id_len=-1, i, k_id;
 	int retval=-1;
-	int rates[] = { 57600, 9600, 4800, 1200, 0 };	/* possible baud rates */
+	int rates[] = { 115200, 57600, 9600, 4800, 1200, 0 };	/* possible baud rates */
 	int rates_idx;
 
 	if (!port)
@@ -1409,58 +1430,6 @@ DECLARE_PROBERIG_BACKEND(kenwood)
 }
 
 
-/* kenwood_init
- *
- * Basically, it sets up *priv
- * REM: serial port is already open (rig->state.rigport.fd)
- */
-int kenwood_init(RIG *rig)
-{
-    const struct rig_caps *caps;
-    const struct kenwood_priv_caps *priv_caps;
-    rig_debug(RIG_DEBUG_TRACE, "%s: called\n", __FUNCTION__);
-
-    if (!rig || !rig->caps)
-        return -RIG_EINVAL;
-
-    caps = rig->caps;
-    if (!caps->priv)
-        return -RIG_ECONF;
-
-    priv_caps = (const struct kenwood_priv_caps *) caps->priv;
-
-#if 0   /* No private data for Kenwood backends */
-    priv = (struct kenwood_priv_data *)malloc(sizeof(struct kenwood_priv_data));
-    if (!priv) {
-        /* whoops! memory shortage! */
-        return -RIG_ENOMEM;
-    }
-
-    rig->state.priv = (void *)priv;
-    /* Assign default values */
-    priv->dummy = -1;  // placeholder for real entries.
-#endif
-
-    return RIG_OK;
-}
-
-/* kenwood_cleanup
- * the serial port is closed by the frontend
- */
-int kenwood_cleanup(RIG *rig)
-{
-    rig_debug(RIG_DEBUG_TRACE, "%s: called\n", __FUNCTION__);
-
-    if (!rig)
-        return -RIG_EINVAL;
-
-    if (rig->state.priv)
-        free(rig->state.priv);
-    rig->state.priv = NULL;
-
-    return RIG_OK;
-}
-
 /*
  * initrigs_kenwood is called by rig_backend_load
  */
@@ -1494,6 +1463,8 @@ DECLARE_INITRIG_BACKEND(kenwood)
 		rig_register(&thf7e_caps);
 		rig_register(&thg71_caps);
 		rig_register(&tmv7_caps);
+		
+		rig_register(&ts480_caps);
 
 		return RIG_OK;
 }
