@@ -1,8 +1,8 @@
 /*
  *  Hamlib Kenwood backend - TH handheld primitives
- *  Copyright (c) 2001-2003 by Stephane Fillod
+ *  Copyright (c) 2001-2005 by Stephane Fillod
  *
- *	$Id: th.c,v 1.20 2004/03/21 18:25:54 f4dwv Exp $
+ *	$Id: th.c,v 1.27 2005/01/25 00:20:33 fillods Exp $
  *
  *   This library is free software; you can redistribute it and/or modify
  *   it under the terms of the GNU Library General Public License as
@@ -33,6 +33,7 @@
 #include "kenwood.h"
 #include "th.h"
 #include "serial.h"
+#include "misc.h"
 
 const struct kenwood_priv_caps  th_priv_caps  = {
     .cmdtrm =  EOM_TH,
@@ -81,7 +82,7 @@ th_decode_event (RIG *rig)
         rmode_t mode;
         int step, shift, rev, tone, ctcss, tonefq, ctcssfq;
 
-        retval = sscanf(asyncbuf, "BUF %d,%"FREQFMT",%d,%d,%d,%d,%d,,%d,,%d,%"FREQFMT",%d",
+        retval = sscanf(asyncbuf, "BUF %d,%"SCNfreq",%d,%d,%d,%d,%d,,%d,,%d,%"SCNfreq",%d",
                                   &vfo, &freq, &step, &shift, &rev, &tone,
                                   &ctcss, &tonefq, &ctcssfq, &offset, &mode);
 
@@ -94,7 +95,7 @@ th_decode_event (RIG *rig)
         vfo = (vfo == 0) ? RIG_VFO_A : RIG_VFO_B;
         mode = (mode == 0) ? RIG_MODE_FM : RIG_MODE_AM;
 
-        rig_debug(RIG_DEBUG_TRACE, "%s: Buffer (vfo %d, freq %lld Hz, mode %d)\n", __FUNCTION__, vfo, freq, mode);
+        rig_debug(RIG_DEBUG_TRACE, "%s: Buffer (vfo %d, freq %"PRIfreq" Hz, mode %d)\n", __FUNCTION__, vfo, freq, mode);
 
         /* Callback execution */
         if (rig->callbacks.vfo_event) {
@@ -193,7 +194,7 @@ th_set_freq (RIG *rig, vfo_t vfo, freq_t freq)
 
     step = 1;
     f=(long long) freq;
-    sprintf(freqbuf, "FQ %011lld,%1d"EOM, f, step);
+    sprintf(freqbuf, "FQ %011"PRIll",%1d"EOM, f, step);
     retval = kenwood_transaction(rig, freqbuf, strlen(freqbuf), ackbuf, &ack_len);
     if (retval != RIG_OK)
         return retval;
@@ -210,7 +211,6 @@ th_get_freq (RIG *rig, vfo_t vfo, freq_t *freq)
 {
     char freqbuf[24], ackbuf[ACKBUF_LEN];
     int retval, step,ack_len=ACKBUF_LEN;
-    long long f;
 
     rig_debug(RIG_DEBUG_TRACE, "%s: called\n", __FUNCTION__);
 
@@ -226,13 +226,12 @@ th_get_freq (RIG *rig, vfo_t vfo, freq_t *freq)
     if (retval != RIG_OK)
         return retval;
 
-    retval = sscanf(ackbuf, "FQ %lld,%d",&f,&step);
+    retval = sscanf(ackbuf, "FQ %"SCNfreq",%d",freq,&step);
     if (retval != 2) {
         rig_debug(RIG_DEBUG_ERR, "%s: Unexpected reply '%s'\n", __FUNCTION__, freqbuf);
         return -RIG_ERJCTED;
     }
 
-    *freq=(freq_t)f;
     return RIG_OK;
 }
 
@@ -717,7 +716,7 @@ th_get_level (RIG *rig, vfo_t vfo, setting_t level, value_t *val)
 			return retval;
 
             retval = sscanf(ackbuf, "PC %d,%d", &v, &l);
-            if (retval != 1 || l < 0 || l > 3) {
+            if (retval != 2 || l < 0 || l > 3) {
                 rig_debug(RIG_DEBUG_ERR, "%s: Unexpected reply '%s'\n", __FUNCTION__, ackbuf);
                 return -RIG_ERJCTED;
             }
@@ -915,12 +914,15 @@ th_get_mem(RIG *rig, vfo_t vfo, int *ch)
 {
 	unsigned char *membuf, ackbuf[ACKBUF_LEN];
 	int retval,ack_len=ACKBUF_LEN;
-	vfo_t tvfo;
+	vfo_t tvfo,cvfo;
 
 	rig_debug(RIG_DEBUG_TRACE, "%s: called\n", __FUNCTION__);
 
-	if(vfo==RIG_VFO_CURR) tvfo=rig->state.current_vfo;
-	else tvfo=vfo;
+	cvfo=rig->state.current_vfo;
+	if(vfo==RIG_VFO_CURR)
+		 tvfo=cvfo;
+	else
+		 tvfo=vfo;
 
 	switch (tvfo) {
 	  case RIG_VFO_VFO:
@@ -953,14 +955,18 @@ th_get_mem(RIG *rig, vfo_t vfo, int *ch)
 	ackbuf[8]='\0';
         *ch = atoi(&ackbuf[5]);
 
+	retval= rig_set_vfo(rig,cvfo);
+	if (retval != RIG_OK)
+		return retval;
+
 	return RIG_OK;
 }
 
 int
 th_set_ptt(RIG *rig, vfo_t vfo, ptt_t ptt)
 {
-        unsigned char *membuf;
-        int retval;
+	unsigned char *membuf, ackbuf[ACKBUF_LEN];
+	int retval,ack_len=ACKBUF_LEN;
 
         rig_debug(RIG_DEBUG_TRACE, "%s: called\n", __FUNCTION__);
 
@@ -975,7 +981,7 @@ th_set_ptt(RIG *rig, vfo_t vfo, ptt_t ptt)
                 return -RIG_EINVAL;
         }
 
-        retval = kenwood_transaction(rig, membuf, strlen(membuf), NULL, NULL);
+        retval = kenwood_transaction(rig, membuf, strlen(membuf), ackbuf, &ack_len);
         if (retval != RIG_OK)
                 return retval;
 
@@ -1008,6 +1014,26 @@ th_set_powerstat(RIG *rig, powerstat_t status)
 	return RIG_OK;
 }
 
+int th_get_powerstat(RIG *rig, powerstat_t *status)
+{
+                unsigned char pwrbuf[50];
+                int pwr_len = 50, retval;
+                                                                                                                            
+                retval = kenwood_transaction (rig, "PS;", 3, pwrbuf, &pwr_len);
+                if (retval != RIG_OK)
+                        return retval;
+                                                                                                                            
+                if (pwr_len != 4) {
+                                rig_debug(RIG_DEBUG_ERR,"kenwood_get_powerstat: wrong answer "
+                                                                "len=%d\n", pwr_len);
+                                return -RIG_ERJCTED;
+                }
+                *status = pwrbuf[2] == '0' ? RIG_POWER_OFF : RIG_POWER_ON;
+                                                                                                                            
+                return RIG_OK;
+}
+                                                                                                                            
+
 int th_get_dcd(RIG *rig, vfo_t vfo, dcd_t *dcd)
 {
         unsigned char *membuf, ackbuf[ACKBUF_LEN];
@@ -1084,7 +1110,7 @@ int th_vfo_op(RIG *rig, vfo_t vfo, vfo_op_t op)
         return RIG_OK;
 }
 
-/* get and set channel tested on thg71 and tmv7	*/
+/* get and set channel tested on thg71   	*/
 /* must work on other th and tm kenwood rigs	*/
 /* Beware : special numbering scheme :		*/
 /* 0-199 : normal memories			*/
@@ -1098,7 +1124,7 @@ int th_get_channel(RIG *rig, channel_t *chan)
 {
     char membuf[64],ackbuf[ACKBUF_LEN];
     int retval,ack_len;
-    long long freq,offset;
+    freq_t freq,offset;
     char req[16],scf[128];
     int step, shift, rev, tone, ctcss, tonefq, ctcssfq;
 
@@ -1135,12 +1161,12 @@ int th_get_channel(RIG *rig, channel_t *chan)
         	return retval;
 
     strcpy(scf,req);
-    strcat(scf,",%"FREQFMT",%d,%d,%d,%d,%d,,%d,,%d,%"FREQFMT);
+    strcat(scf,",%"SCNfreq",%d,%d,%d,%d,%d,,%d,,%d,%"SCNfreq);
     retval = sscanf(ackbuf, scf,
                     &freq, &step, &shift, &rev, &tone,
                     &ctcss, &tonefq, &ctcssfq, &offset);
 
-    chan->freq=(freq_t)freq;
+    chan->freq=freq;
     chan->vfo=RIG_VFO_MEM;
     chan->tuning_step=rig->state.tuning_steps[step].ts;
     if(freq <MHz(136) )  {
@@ -1173,14 +1199,15 @@ int th_get_channel(RIG *rig, channel_t *chan)
 	 chan->ctcss_sql=0;
 
     chan->tx_freq=RIG_FREQ_NONE;
-    if(chan->channel_num<223 && offset==0) {
+    if(chan->channel_num<223 && shift==0) {
 	req[5]='1';
         sprintf(membuf,"%s"EOM,req);
         ack_len=ACKBUF_LEN;
     	retval = kenwood_transaction(rig, membuf, strlen(membuf), ackbuf, &ack_len);
         if (retval == RIG_OK) {
-    		strcat(req,",%"FREQFMT",%d");
-    		retval = sscanf(ackbuf, req, &freq, &step);
+    		strcpy(scf,req);
+    		strcat(scf,",%"SCNfreq",%d");
+    		retval = sscanf(ackbuf, scf, &freq, &step);
 		chan->tx_freq=freq;
 	}
     }
@@ -1270,11 +1297,11 @@ int th_set_channel(RIG *rig, const channel_t *chan)
 	return -RIG_EINVAL;
 
     if(chan->channel_num<=220)
-    sprintf(membuf, "%s,%011lld,%01d,%01d,0,%01d,%01d,,%02d,,%02d,%09lld,0"EOM,
+    sprintf(membuf, "%s,%011"PRIll",%01d,%01d,0,%01d,%01d,,%02d,,%02d,%09"PRIll",0"EOM,
                     req,(long long)freq, step, shift, tone,
                     ctcss, tonefq, ctcssfq, (long long)offset);
     else
-    sprintf(membuf, "%s,%011lld,%01d,%01d,0,%01d,%01d,,%02d,,%02d,%09lld"EOM,
+    sprintf(membuf, "%s,%011"PRIll",%01d,%01d,0,%01d,%01d,,%02d,,%02d,%09"PRIll EOM,
                     req, (long long)freq, step, shift, tone,
                     ctcss, tonefq, ctcssfq, (long long)offset);
 
@@ -1285,7 +1312,7 @@ int th_set_channel(RIG *rig, const channel_t *chan)
 
     if(chan->channel_num<223 && chan->tx_freq!=RIG_FREQ_NONE) {
 	req[5]='1';
-    	sprintf(membuf, "%s,%011lld,%01d"EOM, req,(long long)chan->tx_freq, step);
+    	sprintf(membuf, "%s,%011"PRIll",%01d"EOM, req,(long long)chan->tx_freq, step);
         ack_len=ACKBUF_LEN;
     	retval = kenwood_transaction(rig, membuf, strlen(membuf), ackbuf, &ack_len);
         if (retval != RIG_OK)
