@@ -3,7 +3,7 @@
  * \ingroup hamlib
  * \brief locator and bearing conversion interface
  * \author Stephane Fillod and the Hamlib Group
- * \date 2000-2003
+ * \date 2000-2004
  *
  *  Hamlib Interface - locator, bearing, and conversion calls
  */
@@ -14,7 +14,7 @@
  *  Copyright (c) 2003 by Nate Bargmann
  *  Copyright (c) 2003 by Dave Hines
  *
- *	$Id: locator.c,v 1.14 2003/11/03 15:07:23 n0nb Exp $
+ *	$Id: locator.c,v 1.17 2005/01/18 23:06:34 n0nb Exp $
  *
  *  Code to determine bearing and range was taken from the Great Circle,
  *  by S. R. Sampson, N5OWK.
@@ -72,7 +72,7 @@
 /* arc length for 1 degree, 60 Nautical Miles */
 #define ARC_IN_KM 111.2
 
-/* The following is contributed by Dave Hines
+/* The following is contributed by Dave Hines M1CXW
  *
  * begin dph
  */
@@ -80,25 +80,34 @@
  * These are the constants used when converting between Maidenhead grid
  * locators and longitude/latitude values. MAX_LOCATOR_PAIRS is the maximum
  * number of locator character pairs to convert. This number MUST NOT exceed
- * the number of pairs of values in range[] & weight[].
+ * the number of pairs of values in loc_char_range[].
  * Setting MAX_LOCATOR_PAIRS to 3 will convert the currently defined 6
  * character locators. A value of 4 will convert the extended 8 character
  * locators described in section 3L of "The IARU region 1 VHF managers
  * handbook". Values of 5 and 6 will extent the format even more, to the
- * longest definition I have seen for locators. Beware that there seems to be
- * no universally accepted standard for 10 & 12 character locators.
- * Note that the loc_char_weight values are in minutes of arc, to avoid
- * constants which can't be represented precisely in either binary or decimal.
+ * longest definition I have seen for locators, see
+ *	http://www.btinternet.com/~g8yoa/geog/non-ra.html
+ * Beware that there seems to be no universally accepted standard for 10 & 12
+ * character locators.
+ *
+ * The ranges of characters which will be accepted by locator2longlat, and
+ * generated by longlat2locator, are specified by the loc_char_range[] array.
+ * This array may be changed without requiring any other code changes.
+ *
+ * For the fifth pair to range from aa to xx use:
+ * const static int loc_char_range[] = { 18, 10, 24, 10, 24, 10 };
+ *
+ * For the fifth pair to range from aa to yy use:
+ * const static int loc_char_range[] = { 18, 10, 24, 10, 25, 10 };
  *
  * MAX_LOCATOR_PAIRS now sets the limit locator2longlat() will convert and
  * sets the maximum length longlat2locator() will generate.  Each function
  * properly handles any value from 1 to 6 so MAX_LOCATOR_PAIRS should be
  * left at 6.  MIN_LOCATOR_PAIRS sets a floor on the shortest locator that
  * should be handled.  -N0NB
- *
  */
-const static double loc_char_weight[] = { 600.0, 60.0, 2.5, 0.25, 0.01, 0.001 };
-const static int loc_char_range[] = { 18, 10, 24, 10, 25, 10 };
+const static int loc_char_range[] = { 18, 10, 24, 10, 24, 10 };
+
 #define MAX_LOCATOR_PAIRS       6
 #define MIN_LOCATOR_PAIRS       1
 
@@ -127,7 +136,7 @@ const static int loc_char_range[] = { 18, 10, 24, 10, 25, 10 };
  * \sa dec2dms()
  */
 
-double dms2dec(int degrees, int minutes, double seconds, int sw) {
+double HAMLIB_API dms2dec(int degrees, int minutes, double seconds, int sw) {
 	double st;
 
 	if (degrees < 0)
@@ -167,7 +176,7 @@ double dms2dec(int degrees, int minutes, double seconds, int sw) {
  * \sa dec2dmmm()
  */
 
-double dmmm2dec(int degrees, double minutes, int sw) {
+double HAMLIB_API dmmm2dec(int degrees, double minutes, int sw) {
 	double st;
 
 	if (degrees < 0)
@@ -211,7 +220,7 @@ double dmmm2dec(int degrees, double minutes, int sw) {
  * \sa dms2dec()
  */
 
-int dec2dms(double dec, int *degrees, int *minutes, double *seconds, int *sw) {
+int HAMLIB_API dec2dms(double dec, int *degrees, int *minutes, double *seconds, int *sw) {
 	int deg, min;
 	double st;
 
@@ -287,7 +296,7 @@ int dec2dms(double dec, int *degrees, int *minutes, double *seconds, int *sw) {
  * \sa dmmm2dec()
  */
 
-int dec2dmmm(double dec, int *degrees, double *minutes, int *sw) {
+int HAMLIB_API dec2dmmm(double dec, int *degrees, double *minutes, int *sw) {
 	int r, min;
 	double sec;
 
@@ -319,12 +328,12 @@ int dec2dmmm(double dec, int *degrees, double *minutes, int *sw) {
  *  EM19 will return coordinates equivalent to the southwest corner
  *  of EM19mm.
  *
- * \retval -RIG_EINVAL if locator exceeds RR99xx99yy99 or exceeds length
+ * \retval -RIG_EINVAL if locator exceeds RR99xx99xx99 or exceeds length
  *  limit--currently 1 to 6 lon/lat pairs.
  * \retval RIG_OK if conversion went OK.
  *
- * \bug The fifth pair ranges from aa to yy, there is another convention
- *  that ranges from aa to xx.  At some point both conventions should be
+ * \bug The fifth pair ranges from aa to xx, there is another convention
+ *  that ranges from aa to yy.  At some point both conventions should be
  *  supported.
  *
  * \sa longlat2locator()
@@ -332,10 +341,11 @@ int dec2dmmm(double dec, int *degrees, double *minutes, int *sw) {
 
 /* begin dph */
 
-int locator2longlat(double *longitude, double *latitude, const char *locator) {
+int HAMLIB_API locator2longlat(double *longitude, double *latitude, const char *locator) {
 	int x_or_y, paircount;
 	int locvalue, pair;
-	double xy[2], minutes;
+	int divisions;
+	double xy[2], ordinate;
 
 	/* bail if NULL pointers passed */
 	if (!longitude || !latitude)
@@ -349,9 +359,10 @@ int locator2longlat(double *longitude, double *latitude, const char *locator) {
 	else if (paircount < MIN_LOCATOR_PAIRS)
 		return -RIG_EINVAL;
 
-	/* For x(=lon) and y(=lat) */
+	/* For x(=longitude) and y(=latitude) */
 	for (x_or_y = 0;  x_or_y < 2;  ++x_or_y) {
-		minutes = 0.0;
+		ordinate = -90.0;
+		divisions = 1;
 
 		for (pair = 0;  pair < paircount;  ++pair) {
 			locvalue = locator[pair*2 + x_or_y];
@@ -361,18 +372,19 @@ int locator2longlat(double *longitude, double *latitude, const char *locator) {
 				(isupper(locvalue)) ? 'A' : 'a';
 
 			/* Check range for non-letter/digit or out of range */
-			if (((unsigned) locvalue) >= loc_char_range[pair])
+			if ((locvalue < 0) || (locvalue >= loc_char_range[pair]))
 				return -RIG_EINVAL;
 
-			minutes += locvalue * loc_char_weight[pair];
+			divisions *= loc_char_range[pair];
+			ordinate += locvalue * 180.0 / divisions;
 		}
-		/* Center coordinate */
-		minutes += loc_char_weight[paircount - 1] / 2.0;
+		/* Center ordinate in the Maidenhead "square" or "subsquare" */
+		ordinate += 90.0 / divisions;
 
-		xy[x_or_y] = minutes / 60.0 - 90.0;
+		xy[x_or_y] = ordinate;
 	}
 
-	*longitude = xy[0] * 2;
+	*longitude = xy[0] * 2.0;
 	*latitude = xy[1];
 
 	return RIG_OK;
@@ -387,7 +399,7 @@ int locator2longlat(double *longitude, double *latitude, const char *locator) {
  * \param pair_count	Precision expressed as lon/lat pairs in the locator
  *
  *  Convert longitude/latitude (decimal degrees) to Maidenhead grid locator.
- *  \a locator must point to an array at least \a pair_count * 2 char + '\0'.
+ *  \a locator must point to an array at least \a pair_count * 2 char + '\\0'.
  *
  * \retval -RIG_EINVAL if \a locator is NULL or \a pair_count exceeds
  *  length limit.  Currently 1 to 6 lon/lat pairs.
@@ -403,9 +415,9 @@ int locator2longlat(double *longitude, double *latitude, const char *locator) {
 
 /* begin dph */
 
-int longlat2locator(double longitude, double latitude, char *locator, int pair_count) {
-	int x_or_y, pair, locvalue;
-	double tmp;
+int HAMLIB_API longlat2locator(double longitude, double latitude, char *locator, int pair_count) {
+	int x_or_y, pair, locvalue, divisions;
+	double square_size, ordinate;
 
 	if (!locator)
 		return -RIG_EINVAL;
@@ -414,15 +426,17 @@ int longlat2locator(double longitude, double latitude, char *locator, int pair_c
 		return -RIG_EINVAL;
 
 	for (x_or_y = 0;  x_or_y < 2;  ++x_or_y) {
-		tmp = ((x_or_y == 0) ? longitude / 2. : latitude);
+		ordinate = (x_or_y == 0) ? longitude / 2.0 : latitude;
+		divisions = 1;
 
 		/* The 1e-6 here guards against floating point rounding errors */
-		tmp = fmod(tmp + 270., 180.) * 60. + 1e-6;
+		ordinate = fmod(ordinate + 270.000001, 180.0);
 		for (pair = 0;  pair < pair_count;  ++pair) {
-			locvalue = (int) (tmp / loc_char_weight[pair]);
+			divisions *= loc_char_range[pair];
+			square_size = 180.0 / divisions;
 
-			/* assert(locvalue < loc_char_range[pair]); */
-			tmp -= loc_char_weight[pair] * locvalue;
+			locvalue = (int) (ordinate / square_size);
+			ordinate -= square_size * locvalue;
 			locvalue += (loc_char_range[pair] == 10) ? '0':'A';
 			locator[pair * 2 + x_or_y] = locvalue;
 		}
@@ -460,7 +474,7 @@ int longlat2locator(double longitude, double latitude, char *locator, int pair_c
  * \sa distance_long_path(), azimuth_long_path()
  */
 
-int qrb(double lon1, double lat1, double lon2, double lat2, double *distance, double *azimuth) {
+int HAMLIB_API qrb(double lon1, double lat1, double lon2, double lat2, double *distance, double *azimuth) {
 	double delta_long, tmp, arc, az;
 
 	/* bail if NULL pointers passed */
@@ -559,7 +573,7 @@ int qrb(double lon1, double lat1, double lon2, double lat2, double *distance, do
  * \sa qrb()
  */
 
-double distance_long_path(double distance) {
+double HAMLIB_API distance_long_path(double distance) {
 	 return (ARC_IN_KM * 360.0) - distance;
 }
 
@@ -575,6 +589,6 @@ double distance_long_path(double distance) {
  * \sa qrb()
  */
 
-double azimuth_long_path(double azimuth) {
+double HAMLIB_API azimuth_long_path(double azimuth) {
 	return 360.0 - azimuth;
 }
