@@ -1,8 +1,8 @@
 /*
  *  Hamlib Interface - main file
- *  Copyright (c) 2000-2003 by Stephane Fillod and Frank Singleton
+ *  Copyright (c) 2000-2005 by Stephane Fillod and Frank Singleton
  *
- *	$Id: rig.c,v 1.79 2003/11/19 07:32:10 fillods Exp $
+ *	$Id: rig.c,v 1.88 2005/04/03 22:33:08 fillods Exp $
  *
  *   This library is free software; you can redistribute it and/or modify
  *   it under the terms of the GNU Library General Public License as
@@ -26,7 +26,7 @@
  * \brief Ham Radio Control Libraries interface
  * \author Stephane Fillod
  * \author Frank Singleton
- * \date 2000-2003
+ * \date 2000-2005
  *
  * Hamlib interface is a frontend implementing wrapper functions.
  */
@@ -56,8 +56,9 @@
 #include <fcntl.h>
 
 
-#include <hamlib/rig.h>
-#include <serial.h>
+#include "hamlib/rig.h"
+#include "serial.h"
+#include "parallel.h"
 #include "event.h"
 
 /**
@@ -69,7 +70,7 @@ const char hamlib_version[] = "Hamlib version " PACKAGE_VERSION;
  * \brief Hamlib copyright notice
  */
 const char hamlib_copyright[] = 
-  "Copyright (C) 2000-2003 Stephane Fillod and Frank Singleton\n"
+  "Copyright (C) 2000-2005 Stephane Fillod and Frank Singleton\n"
   "This is free software; see the source for copying conditions.  There is NO\n"
   "warranty; not even for MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.";
 
@@ -84,6 +85,8 @@ const char hamlib_copyright[] =
 
 #if defined(WIN32)
 #define DEFAULT_PARALLEL_PORT "\\\\.\\$VDMLPT1"
+#elif defined(HAVE_DEV_PPBUS_PPI_H)
+#define DEFAULT_PARALLEL_PORT "/dev/ppi0"
 #else
 #define DEFAULT_PARALLEL_PORT "/dev/parport0"
 #endif
@@ -206,7 +209,7 @@ int foreach_opened_rig(int (*cfunc)(RIG *, rig_ptr_t), rig_ptr_t data)
  *
  * \todo support gettext/localization
  */
-const char *rigerror(int errnum)
+const char * HAMLIB_API rigerror(int errnum)
 {
 		errnum = abs(errnum);
 		if (errnum > ERROR_TBL_SZ)
@@ -227,7 +230,7 @@ const char *rigerror(int errnum)
  * \sa rig_cleanup(), rig_open()
  */
 
-RIG *rig_init(rig_model_t rig_model)
+RIG * HAMLIB_API rig_init(rig_model_t rig_model)
 {
 		RIG *rig;
 		const struct rig_caps *caps;
@@ -392,7 +395,7 @@ RIG *rig_init(rig_model_t rig_model)
  * \sa rig_init(), rig_close()
  */
 
-int rig_open(RIG *rig)
+int HAMLIB_API rig_open(RIG *rig)
 {
 		const struct rig_caps *caps;
 		struct rig_state *rs;
@@ -553,7 +556,7 @@ int rig_open(RIG *rig)
  * \sa rig_cleanup(), rig_open()
  */
 
-int rig_close(RIG *rig)
+int HAMLIB_API rig_close(RIG *rig)
 {
 	const struct rig_caps *caps;
 	struct rig_state *rs;
@@ -654,7 +657,7 @@ int rig_close(RIG *rig)
  * \sa rig_init(), rig_close()
  */
 
-int rig_cleanup(RIG *rig)
+int HAMLIB_API rig_cleanup(RIG *rig)
 {
 		rig_debug(RIG_DEBUG_VERBOSE,"rig:rig_cleanup called \n");
 
@@ -694,7 +697,7 @@ int rig_cleanup(RIG *rig)
  * \sa rig_get_freq()
  */
 
-int rig_set_freq(RIG *rig, vfo_t vfo, freq_t freq)
+int HAMLIB_API rig_set_freq(RIG *rig, vfo_t vfo, freq_t freq)
 {
 	const struct rig_caps *caps;
 	int retcode;
@@ -749,7 +752,7 @@ int rig_set_freq(RIG *rig, vfo_t vfo, freq_t freq)
  * \sa rig_set_freq()
  */
 
-int rig_get_freq(RIG *rig, vfo_t vfo, freq_t *freq)
+int HAMLIB_API rig_get_freq(RIG *rig, vfo_t vfo, freq_t *freq)
 {
 	const struct rig_caps *caps;
 	int retcode;
@@ -805,7 +808,7 @@ int rig_get_freq(RIG *rig, vfo_t vfo, freq_t *freq)
  * \sa rig_get_mode()
  */
 
-int rig_set_mode(RIG *rig, vfo_t vfo, rmode_t mode, pbwidth_t width)
+int HAMLIB_API rig_set_mode(RIG *rig, vfo_t vfo, rmode_t mode, pbwidth_t width)
 {
 	const struct rig_caps *caps;
 	int retcode;
@@ -819,7 +822,7 @@ int rig_set_mode(RIG *rig, vfo_t vfo, rmode_t mode, pbwidth_t width)
 	if (caps->set_mode == NULL)
 		return -RIG_ENAVAIL;
 
-	if ((caps->targetable_vfo&RIG_TARGETABLE_ALL) ||
+	if ((caps->targetable_vfo&RIG_TARGETABLE_MODE) ||
 			vfo == RIG_VFO_CURR || vfo == rig->state.current_vfo) {
 		retcode = caps->set_mode(rig, vfo, mode, width);
 	} else {
@@ -864,7 +867,7 @@ int rig_set_mode(RIG *rig, vfo_t vfo, rmode_t mode, pbwidth_t width)
  * \sa rig_set_mode()
  */
 
-int rig_get_mode(RIG *rig, vfo_t vfo, rmode_t *mode, pbwidth_t *width)
+int HAMLIB_API rig_get_mode(RIG *rig, vfo_t vfo, rmode_t *mode, pbwidth_t *width)
 {
 	const struct rig_caps *caps;
 	int retcode;
@@ -878,7 +881,7 @@ int rig_get_mode(RIG *rig, vfo_t vfo, rmode_t *mode, pbwidth_t *width)
 	if (caps->get_mode == NULL)
 		return -RIG_ENAVAIL;
 
-	if ((caps->targetable_vfo&RIG_TARGETABLE_ALL) || 
+	if ((caps->targetable_vfo&RIG_TARGETABLE_MODE) || 
 			vfo == RIG_VFO_CURR || vfo == rig->state.current_vfo) {
 		retcode = caps->get_mode(rig, vfo, mode, width);
 	} else {
@@ -916,7 +919,7 @@ int rig_get_mode(RIG *rig, vfo_t vfo, rmode_t *mode, pbwidth_t *width)
  * \sa rig_passband_narrow(), rig_passband_wide()
  */
 
-pbwidth_t rig_passband_normal(RIG *rig, rmode_t mode)
+pbwidth_t HAMLIB_API rig_passband_normal(RIG *rig, rmode_t mode)
 {
 		const struct rig_state *rs;
 		int i;
@@ -950,7 +953,7 @@ pbwidth_t rig_passband_normal(RIG *rig, rmode_t mode)
  * \sa rig_passband_normal(), rig_passband_wide()
  */
 
-pbwidth_t rig_passband_narrow(RIG *rig, rmode_t mode)
+pbwidth_t HAMLIB_API rig_passband_narrow(RIG *rig, rmode_t mode)
 {
 		const struct rig_state *rs;
 		pbwidth_t normal;
@@ -992,7 +995,7 @@ pbwidth_t rig_passband_narrow(RIG *rig, rmode_t mode)
  * \sa rig_passband_narrow(), rig_passband_normal()
  */
 
-pbwidth_t rig_passband_wide(RIG *rig, rmode_t mode)
+pbwidth_t HAMLIB_API rig_passband_wide(RIG *rig, rmode_t mode)
 {
 		const struct rig_state *rs;
 		pbwidth_t normal;
@@ -1035,7 +1038,7 @@ pbwidth_t rig_passband_wide(RIG *rig, rmode_t mode)
  * \sa rig_get_vfo()
  */
 
-int rig_set_vfo(RIG *rig, vfo_t vfo)
+int HAMLIB_API rig_set_vfo(RIG *rig, vfo_t vfo)
 {
 		const struct rig_caps *caps;
 		int retcode;
@@ -1070,7 +1073,7 @@ int rig_set_vfo(RIG *rig, vfo_t vfo)
  * \sa rig_set_vfo()
  */
 
-int rig_get_vfo(RIG *rig, vfo_t *vfo)
+int HAMLIB_API rig_get_vfo(RIG *rig, vfo_t *vfo)
 {
 		const struct rig_caps *caps;
 		int retcode;
@@ -1103,7 +1106,7 @@ int rig_get_vfo(RIG *rig, vfo_t *vfo)
  *
  * \sa rig_get_ptt()
  */
-int rig_set_ptt(RIG *rig, vfo_t vfo, ptt_t ptt)
+int HAMLIB_API rig_set_ptt(RIG *rig, vfo_t vfo, ptt_t ptt)
 {
 		const struct rig_caps *caps;
 		int retcode;
@@ -1119,7 +1122,7 @@ int rig_set_ptt(RIG *rig, vfo_t vfo, ptt_t ptt)
 			if (caps->set_ptt == NULL)
 				return -RIG_ENIMPL;
 
-			if ((caps->targetable_vfo&RIG_TARGETABLE_ALL) || 
+			if ((caps->targetable_vfo&RIG_TARGETABLE_PURE) || 
 						vfo == RIG_VFO_CURR || vfo == rig->state.current_vfo)
 				return caps->set_ptt(rig, vfo, ptt);
 	
@@ -1137,12 +1140,13 @@ int rig_set_ptt(RIG *rig, vfo_t vfo, ptt_t ptt)
 			break;
 
 		case RIG_PTT_SERIAL_DTR:
+	                        return ser_set_dtr(&rig->state.pttport, ptt==RIG_PTT_ON);
+
 		case RIG_PTT_SERIAL_RTS:
-				ser_ptt_set(&rig->state.pttport, ptt);
-				break;
+	                        return ser_set_rts(&rig->state.pttport, ptt==RIG_PTT_ON);
+
 		case RIG_PTT_PARALLEL:
-				par_ptt_set(&rig->state.pttport, ptt);
-				break;
+				return par_ptt_set(&rig->state.pttport, ptt);
 
 		case RIG_PTT_NONE:
 				return -RIG_ENAVAIL;	/* not available */
@@ -1167,10 +1171,10 @@ int rig_set_ptt(RIG *rig, vfo_t vfo, ptt_t ptt)
  *
  * \sa rig_set_ptt()
  */
-int rig_get_ptt(RIG *rig, vfo_t vfo, ptt_t *ptt)
+int HAMLIB_API rig_get_ptt(RIG *rig, vfo_t vfo, ptt_t *ptt)
 {	
 		const struct rig_caps *caps;
-		int retcode;
+		int retcode, status;
 		vfo_t curr_vfo;
 
 		if (CHECK_RIG_ARG(rig) || !ptt)
@@ -1183,7 +1187,7 @@ int rig_get_ptt(RIG *rig, vfo_t vfo, ptt_t *ptt)
 			if (caps->get_ptt == NULL)
 				return -RIG_ENIMPL;
 
-			if ((caps->targetable_vfo&RIG_TARGETABLE_ALL) || 
+			if ((caps->targetable_vfo&RIG_TARGETABLE_PURE) || 
 						vfo == RIG_VFO_CURR || vfo == rig->state.current_vfo)
 				return caps->get_ptt(rig, vfo, ptt);
 	
@@ -1201,12 +1205,17 @@ int rig_get_ptt(RIG *rig, vfo_t vfo, ptt_t *ptt)
 			break;
 
 		case RIG_PTT_SERIAL_RTS:
+			retcode = ser_get_rts(&rig->state.pttport, &status);
+			*ptt = status ? RIG_PTT_ON : RIG_PTT_OFF;
+			return retcode;
+
 		case RIG_PTT_SERIAL_DTR:
-				ser_ptt_get(&rig->state.pttport, ptt);
-				break;
+			retcode = ser_get_dtr(&rig->state.pttport, &status);
+			*ptt = status ? RIG_PTT_ON : RIG_PTT_OFF;
+			return retcode;
+
 		case RIG_PTT_PARALLEL:
-				par_ptt_get(&rig->state.pttport, ptt);
-				break;
+				return par_ptt_get(&rig->state.pttport, ptt);
 
 		case RIG_PTT_NONE:
 				return -RIG_ENAVAIL;	/* not available */
@@ -1231,10 +1240,10 @@ int rig_get_ptt(RIG *rig, vfo_t vfo, ptt_t *ptt)
  * set appropriately).
  *
  */
-int rig_get_dcd(RIG *rig, vfo_t vfo, dcd_t *dcd)
+int HAMLIB_API rig_get_dcd(RIG *rig, vfo_t vfo, dcd_t *dcd)
 {	
 		const struct rig_caps *caps;
-		int retcode;
+		int retcode, status;
 		vfo_t curr_vfo;
 
 		if (CHECK_RIG_ARG(rig) || !dcd)
@@ -1247,7 +1256,7 @@ int rig_get_dcd(RIG *rig, vfo_t vfo, dcd_t *dcd)
 			if (caps->get_dcd == NULL)
 				return -RIG_ENIMPL;
 
-			if ((caps->targetable_vfo&RIG_TARGETABLE_ALL) || 
+			if ((caps->targetable_vfo&RIG_TARGETABLE_PURE) || 
 						vfo == RIG_VFO_CURR || vfo == rig->state.current_vfo)
 				return caps->get_dcd(rig, vfo, dcd);
 	
@@ -1265,13 +1274,23 @@ int rig_get_dcd(RIG *rig, vfo_t vfo, dcd_t *dcd)
 			break;
 
 		case RIG_DCD_SERIAL_CTS:
+			retcode = ser_get_cts(&rig->state.dcdport, &status);
+			*dcd = status ? RIG_DCD_ON : RIG_DCD_OFF;
+			return retcode;
+
 		case RIG_DCD_SERIAL_DSR:
+			retcode = ser_get_dsr(&rig->state.dcdport, &status);
+			*dcd = status ? RIG_DCD_ON : RIG_DCD_OFF;
+			return retcode;
+
 		case RIG_DCD_SERIAL_CAR:
-				ser_dcd_get(&rig->state.dcdport, dcd);
-				break;
+			retcode = ser_get_car(&rig->state.dcdport, &status);
+			*dcd = status ? RIG_DCD_ON : RIG_DCD_OFF;
+			return retcode;
+
+			
 		case RIG_DCD_PARALLEL:
-				par_dcd_get(&rig->state.dcdport, dcd);
-				break;
+			return par_dcd_get(&rig->state.dcdport, dcd);
 
 		case RIG_DCD_NONE:
 				return -RIG_ENAVAIL;	/* not available */
@@ -1298,7 +1317,7 @@ int rig_get_dcd(RIG *rig, vfo_t vfo, dcd_t *dcd)
  *
  * \sa rig_get_rptr_shift()
  */
-int rig_set_rptr_shift(RIG *rig, vfo_t vfo, rptr_shift_t rptr_shift)
+int HAMLIB_API rig_set_rptr_shift(RIG *rig, vfo_t vfo, rptr_shift_t rptr_shift)
 {
 		const struct rig_caps *caps;
 		int retcode;
@@ -1312,7 +1331,7 @@ int rig_set_rptr_shift(RIG *rig, vfo_t vfo, rptr_shift_t rptr_shift)
 		if (caps->set_rptr_shift == NULL)
 			return -RIG_ENAVAIL;
 
-		if ((caps->targetable_vfo&RIG_TARGETABLE_ALL) || 
+		if ((caps->targetable_vfo&RIG_TARGETABLE_PURE) || 
 						vfo == RIG_VFO_CURR || vfo == rig->state.current_vfo)
 			return caps->set_rptr_shift(rig, vfo, rptr_shift);
 
@@ -1342,7 +1361,7 @@ int rig_set_rptr_shift(RIG *rig, vfo_t vfo, rptr_shift_t rptr_shift)
  *
  * \sa rig_set_rptr_shift()
  */
-int rig_get_rptr_shift(RIG *rig, vfo_t vfo, rptr_shift_t *rptr_shift)
+int HAMLIB_API rig_get_rptr_shift(RIG *rig, vfo_t vfo, rptr_shift_t *rptr_shift)
 {
 		const struct rig_caps *caps;
 		int retcode;
@@ -1356,7 +1375,7 @@ int rig_get_rptr_shift(RIG *rig, vfo_t vfo, rptr_shift_t *rptr_shift)
 		if (caps->get_rptr_shift == NULL)
 			return -RIG_ENAVAIL;
 
-		if ((caps->targetable_vfo&RIG_TARGETABLE_ALL) || 
+		if ((caps->targetable_vfo&RIG_TARGETABLE_PURE) || 
 						vfo == RIG_VFO_CURR || vfo == rig->state.current_vfo)
 			return caps->get_rptr_shift(rig, vfo, rptr_shift);
 
@@ -1387,7 +1406,7 @@ int rig_get_rptr_shift(RIG *rig, vfo_t vfo, rptr_shift_t *rptr_shift)
  * \sa rig_get_rptr_offs()
  */
 
-int rig_set_rptr_offs(RIG *rig, vfo_t vfo, shortfreq_t rptr_offs)
+int HAMLIB_API rig_set_rptr_offs(RIG *rig, vfo_t vfo, shortfreq_t rptr_offs)
 {
 		const struct rig_caps *caps;
 		int retcode;
@@ -1401,7 +1420,7 @@ int rig_set_rptr_offs(RIG *rig, vfo_t vfo, shortfreq_t rptr_offs)
 		if (caps->set_rptr_offs == NULL)
 			return -RIG_ENAVAIL;
 
-		if ((caps->targetable_vfo&RIG_TARGETABLE_ALL) || 
+		if ((caps->targetable_vfo&RIG_TARGETABLE_PURE) || 
 						vfo == RIG_VFO_CURR || vfo == rig->state.current_vfo)
 			return caps->set_rptr_offs(rig, vfo, rptr_offs);
 
@@ -1432,7 +1451,7 @@ int rig_set_rptr_offs(RIG *rig, vfo_t vfo, shortfreq_t rptr_offs)
  * \sa rig_set_rptr_offs()
  */
 
-int rig_get_rptr_offs(RIG *rig, vfo_t vfo, shortfreq_t *rptr_offs)
+int HAMLIB_API rig_get_rptr_offs(RIG *rig, vfo_t vfo, shortfreq_t *rptr_offs)
 {
 		const struct rig_caps *caps;
 		int retcode;
@@ -1446,7 +1465,7 @@ int rig_get_rptr_offs(RIG *rig, vfo_t vfo, shortfreq_t *rptr_offs)
 		if (caps->get_rptr_offs == NULL)
 			return -RIG_ENAVAIL;
 
-		if ((caps->targetable_vfo&RIG_TARGETABLE_ALL) || 
+		if ((caps->targetable_vfo&RIG_TARGETABLE_PURE) || 
 						vfo == RIG_VFO_CURR || vfo == rig->state.current_vfo)
 			return caps->get_rptr_offs(rig, vfo, rptr_offs);
 
@@ -1469,16 +1488,16 @@ int rig_get_rptr_offs(RIG *rig, vfo_t vfo, shortfreq_t *rptr_offs)
  * \param vfo	The target VFO
  * \param tx_freq	The transmit split frequency to set to
  *
- *  Sets the split frequencies.
+ *  Sets the split(TX) frequency.
  *
  * \return RIG_OK if the operation has been sucessful, otherwise 
  * a negative value if an error occured (in which case, cause is 
  * set appropriately).
  *
- * \sa rig_get_split_freq()
+ * \sa rig_get_split_freq(), rig_set_split_vfo()
  */
 
-int rig_set_split_freq(RIG *rig, vfo_t vfo, freq_t tx_freq)
+int HAMLIB_API rig_set_split_freq(RIG *rig, vfo_t vfo, freq_t tx_freq)
 {
 		const struct rig_caps *caps;
 		int retcode;
@@ -1489,22 +1508,32 @@ int rig_set_split_freq(RIG *rig, vfo_t vfo, freq_t tx_freq)
 
 		caps = rig->caps;
 
-		if (caps->set_split_freq == NULL)
-			return -RIG_ENAVAIL;
-
-		if ((caps->targetable_vfo&RIG_TARGETABLE_ALL) || 
-						vfo == RIG_VFO_CURR || vfo == rig->state.current_vfo)
+		if (caps->set_split_freq && 
+				((caps->targetable_vfo&RIG_TARGETABLE_PURE) || 
+				vfo == RIG_VFO_CURR || vfo == rig->state.current_vfo))
 			return caps->set_split_freq(rig, vfo, tx_freq);
 
-		if (!caps->set_vfo)
-			return -RIG_ENTARGET;
 		curr_vfo = rig->state.current_vfo;
-		retcode = caps->set_vfo(rig, vfo);
+		if (caps->set_vfo) {
+			retcode = caps->set_vfo(rig, vfo);
+		} else if (rig_has_vfo_op(rig, RIG_OP_TOGGLE) && caps->vfo_op) {
+			retcode = caps->vfo_op(rig, vfo, RIG_OP_TOGGLE);
+		} else {
+			return -RIG_ENAVAIL;
+		}
 		if (retcode != RIG_OK)
-				return retcode;
+			return retcode;
 
-		retcode = caps->set_split_freq(rig, vfo, tx_freq);
-		caps->set_vfo(rig, curr_vfo);
+		if (caps->set_split_freq)
+			retcode = caps->set_split_freq(rig, vfo, tx_freq);
+		else
+			retcode = caps->set_freq(rig, RIG_VFO_CURR, tx_freq);
+
+		if (caps->set_vfo) {
+			caps->set_vfo(rig, curr_vfo);
+		} else {
+			caps->vfo_op(rig, vfo, RIG_OP_TOGGLE);
+		}
 		return retcode;
 }
 
@@ -1514,7 +1543,7 @@ int rig_set_split_freq(RIG *rig, vfo_t vfo, freq_t tx_freq)
  * \param vfo	The target VFO
  * \param tx_freq	The location where to store the current transmit split frequency
  *
- *  Retrieves the current split frequencies.
+ *  Retrieves the current split(TX) frequency.
  *
  * \return RIG_OK if the operation has been sucessful, otherwise 
  * a negative value if an error occured (in which case, cause is 
@@ -1522,7 +1551,7 @@ int rig_set_split_freq(RIG *rig, vfo_t vfo, freq_t tx_freq)
  *
  * \sa rig_set_split_freq()
  */
-int rig_get_split_freq(RIG *rig, vfo_t vfo, freq_t *tx_freq)
+int HAMLIB_API rig_get_split_freq(RIG *rig, vfo_t vfo, freq_t *tx_freq)
 {
 		const struct rig_caps *caps;
 		int retcode;
@@ -1533,22 +1562,33 @@ int rig_get_split_freq(RIG *rig, vfo_t vfo, freq_t *tx_freq)
 
 		caps = rig->caps;
 
-		if (caps->get_split_freq == NULL)
-			return -RIG_ENAVAIL;
-
-		if ((caps->targetable_vfo&RIG_TARGETABLE_ALL) || 
-						vfo == RIG_VFO_CURR || vfo == rig->state.current_vfo)
+		if (caps->get_split_freq && 
+				((caps->targetable_vfo&RIG_TARGETABLE_PURE) || 
+				vfo == RIG_VFO_CURR || vfo == rig->state.current_vfo))
 			return caps->get_split_freq(rig, vfo, tx_freq);
 
-		if (!caps->set_vfo)
-			return -RIG_ENTARGET;
 		curr_vfo = rig->state.current_vfo;
-		retcode = caps->set_vfo(rig, vfo);
+		if (caps->set_vfo) {
+			retcode = caps->set_vfo(rig, vfo);
+		} else if (rig_has_vfo_op(rig, RIG_OP_TOGGLE) && caps->vfo_op) {
+			retcode = caps->vfo_op(rig, vfo, RIG_OP_TOGGLE);
+		} else {
+			return -RIG_ENAVAIL;
+		}
 		if (retcode != RIG_OK)
-				return retcode;
+			return retcode;
+
+		if (caps->get_split_freq)
+			retcode = caps->get_split_freq(rig, vfo, tx_freq);
+		else
+			retcode = caps->get_freq(rig, RIG_VFO_CURR, tx_freq);
+
+		if (caps->set_vfo) {
+			caps->set_vfo(rig, curr_vfo);
+		} else {
+			caps->vfo_op(rig, vfo, RIG_OP_TOGGLE);
+		}
 
-		retcode = caps->get_split_freq(rig, vfo, tx_freq);
-		caps->set_vfo(rig, curr_vfo);
 		return retcode;
 }
 
@@ -1559,7 +1599,7 @@ int rig_get_split_freq(RIG *rig, vfo_t vfo, freq_t *tx_freq)
  * \param tx_mode	The transmit split mode to set to
  * \param tx_width	The transmit split width to set to
  *
- *  Sets the split mode.
+ *  Sets the split(TX) mode.
  *
  * \return RIG_OK if the operation has been sucessful, otherwise 
  * a negative value if an error occured (in which case, cause is 
@@ -1568,7 +1608,7 @@ int rig_get_split_freq(RIG *rig, vfo_t vfo, freq_t *tx_freq)
  * \sa rig_get_split_mode()
  */
 
-int rig_set_split_mode(RIG *rig, vfo_t vfo, rmode_t tx_mode, pbwidth_t tx_width)
+int HAMLIB_API rig_set_split_mode(RIG *rig, vfo_t vfo, rmode_t tx_mode, pbwidth_t tx_width)
 {
 		const struct rig_caps *caps;
 		int retcode;
@@ -1579,22 +1619,33 @@ int rig_set_split_mode(RIG *rig, vfo_t vfo, rmode_t tx_mode, pbwidth_t tx_width)
 
 		caps = rig->caps;
 
-		if (caps->set_split_mode == NULL)
-			return -RIG_ENAVAIL;
-
-		if ((caps->targetable_vfo&RIG_TARGETABLE_ALL) || 
-						vfo == RIG_VFO_CURR || vfo == rig->state.current_vfo)
+		if (caps->set_split_mode && 
+				((caps->targetable_vfo&RIG_TARGETABLE_PURE) || 
+				vfo == RIG_VFO_CURR || vfo == rig->state.current_vfo))
 			return caps->set_split_mode(rig, vfo, tx_mode, tx_width);
 
-		if (!caps->set_vfo)
-			return -RIG_ENTARGET;
 		curr_vfo = rig->state.current_vfo;
-		retcode = caps->set_vfo(rig, vfo);
+		if (caps->set_vfo) {
+			retcode = caps->set_vfo(rig, vfo);
+		} else if (rig_has_vfo_op(rig, RIG_OP_TOGGLE) && caps->vfo_op) {
+			retcode = caps->vfo_op(rig, vfo, RIG_OP_TOGGLE);
+		} else {
+			return -RIG_ENAVAIL;
+		}
 		if (retcode != RIG_OK)
-				return retcode;
+			return retcode;
+
+		if (caps->set_split_mode)
+			retcode = caps->set_split_mode(rig, vfo, tx_mode, tx_width);
+		else
+			retcode = caps->set_mode(rig, RIG_VFO_CURR, tx_mode, tx_width);
+
+		if (caps->set_vfo) {
+			caps->set_vfo(rig, curr_vfo);
+		} else {
+			caps->vfo_op(rig, vfo, RIG_OP_TOGGLE);
+		}
 
-		retcode = caps->set_split_mode(rig, vfo, tx_mode, tx_width);
-		caps->set_vfo(rig, curr_vfo);
 		return retcode;
 }
 
@@ -1605,7 +1656,7 @@ int rig_set_split_mode(RIG *rig, vfo_t vfo, rmode_t tx_mode, pbwidth_t tx_width)
  * \param tx_mode	The location where to store the current transmit split mode
  * \param tx_width	The location where to store the current transmit split width
  *
- *  Retrieves the current split mode.
+ *  Retrieves the current split(TX) mode.
  *
  * \return RIG_OK if the operation has been sucessful, otherwise 
  * a negative value if an error occured (in which case, cause is 
@@ -1613,7 +1664,7 @@ int rig_set_split_mode(RIG *rig, vfo_t vfo, rmode_t tx_mode, pbwidth_t tx_width)
  *
  * \sa rig_set_split_mode()
  */
-int rig_get_split_mode(RIG *rig, vfo_t vfo, rmode_t *tx_mode, pbwidth_t *tx_width)
+int HAMLIB_API rig_get_split_mode(RIG *rig, vfo_t vfo, rmode_t *tx_mode, pbwidth_t *tx_width)
 {
 		const struct rig_caps *caps;
 		int retcode;
@@ -1624,22 +1675,32 @@ int rig_get_split_mode(RIG *rig, vfo_t vfo, rmode_t *tx_mode, pbwidth_t *tx_widt
 
 		caps = rig->caps;
 
-		if (caps->get_split_mode == NULL)
-			return -RIG_ENAVAIL;
-
-		if ((caps->targetable_vfo&RIG_TARGETABLE_ALL) || 
-						vfo == RIG_VFO_CURR || vfo == rig->state.current_vfo)
+		if (caps->get_split_mode && 
+				 ((caps->targetable_vfo&RIG_TARGETABLE_PURE) || 
+				vfo == RIG_VFO_CURR || vfo == rig->state.current_vfo))
 			return caps->get_split_mode(rig, vfo, tx_mode, tx_width);
 
-		if (!caps->set_vfo)
-			return -RIG_ENTARGET;
 		curr_vfo = rig->state.current_vfo;
-		retcode = caps->set_vfo(rig, vfo);
+		if (caps->set_vfo) {
+			retcode = caps->set_vfo(rig, vfo);
+		} else if (rig_has_vfo_op(rig, RIG_OP_TOGGLE) && caps->vfo_op) {
+			retcode = caps->vfo_op(rig, vfo, RIG_OP_TOGGLE);
+		} else {
+			return -RIG_ENAVAIL;
+		}
 		if (retcode != RIG_OK)
-				return retcode;
+			return retcode;
 
-		retcode = caps->get_split_mode(rig, vfo, tx_mode, tx_width);
-		caps->set_vfo(rig, curr_vfo);
+		if (caps->get_split_mode)
+			retcode = caps->get_split_mode(rig, vfo, tx_mode, tx_width);
+		else
+			retcode = caps->get_mode(rig, RIG_VFO_CURR, tx_mode, tx_width);
+
+		if (caps->set_vfo) {
+			caps->set_vfo(rig, curr_vfo);
+		} else {
+			caps->vfo_op(rig, vfo, RIG_OP_TOGGLE);
+		}
 		return retcode;
 }
 
@@ -1659,7 +1720,7 @@ int rig_get_split_mode(RIG *rig, vfo_t vfo, rmode_t *tx_mode, pbwidth_t *tx_widt
  *
  * \sa rig_get_split_vfo()
  */
-int rig_set_split_vfo(RIG *rig, vfo_t vfo, split_t split, vfo_t tx_vfo)
+int HAMLIB_API rig_set_split_vfo(RIG *rig, vfo_t vfo, split_t split, vfo_t tx_vfo)
 {
 		const struct rig_caps *caps;
 		int retcode;
@@ -1673,7 +1734,7 @@ int rig_set_split_vfo(RIG *rig, vfo_t vfo, split_t split, vfo_t tx_vfo)
 		if (caps->set_split_vfo == NULL)
 			return -RIG_ENAVAIL;
 
-		if ((caps->targetable_vfo&RIG_TARGETABLE_ALL) || 
+		if ((caps->targetable_vfo&RIG_TARGETABLE_PURE) || 
 						vfo == RIG_VFO_CURR || vfo == rig->state.current_vfo)
 			return caps->set_split_vfo(rig, vfo, split, tx_vfo);
 
@@ -1704,7 +1765,7 @@ int rig_set_split_vfo(RIG *rig, vfo_t vfo, split_t split, vfo_t tx_vfo)
  *
  * \sa rig_set_split_vfo()
  */
-int rig_get_split_vfo(RIG *rig, vfo_t vfo, split_t *split, vfo_t *tx_vfo)
+int HAMLIB_API rig_get_split_vfo(RIG *rig, vfo_t vfo, split_t *split, vfo_t *tx_vfo)
 {
 		const struct rig_caps *caps;
 		int retcode;
@@ -1718,7 +1779,7 @@ int rig_get_split_vfo(RIG *rig, vfo_t vfo, split_t *split, vfo_t *tx_vfo)
 		if (caps->get_split_vfo == NULL)
 			return -RIG_ENAVAIL;
 
-		if ((caps->targetable_vfo&RIG_TARGETABLE_ALL) || 
+		if ((caps->targetable_vfo&RIG_TARGETABLE_PURE) || 
 						vfo == RIG_VFO_CURR || vfo == rig->state.current_vfo)
 			return caps->get_split_vfo(rig, vfo, split, tx_vfo);
 
@@ -1749,7 +1810,7 @@ int rig_get_split_vfo(RIG *rig, vfo_t vfo, split_t *split, vfo_t *tx_vfo)
  * \sa rig_get_rit()
  */
 
-int rig_set_rit(RIG *rig, vfo_t vfo, shortfreq_t rit)
+int HAMLIB_API rig_set_rit(RIG *rig, vfo_t vfo, shortfreq_t rit)
 {
 		const struct rig_caps *caps;
 		int retcode;
@@ -1763,7 +1824,7 @@ int rig_set_rit(RIG *rig, vfo_t vfo, shortfreq_t rit)
 		if (caps->set_rit == NULL)
 			return -RIG_ENAVAIL;
 
-		if ((caps->targetable_vfo&RIG_TARGETABLE_ALL) || 
+		if ((caps->targetable_vfo&RIG_TARGETABLE_PURE) || 
 						vfo == RIG_VFO_CURR || vfo == rig->state.current_vfo)
 			return caps->set_rit(rig, vfo, rit);
 
@@ -1794,7 +1855,7 @@ int rig_set_rit(RIG *rig, vfo_t vfo, shortfreq_t rit)
  * \sa rig_set_rit()
  */
 
-int rig_get_rit(RIG *rig, vfo_t vfo, shortfreq_t *rit)
+int HAMLIB_API rig_get_rit(RIG *rig, vfo_t vfo, shortfreq_t *rit)
 {
 		const struct rig_caps *caps;
 		int retcode;
@@ -1808,7 +1869,7 @@ int rig_get_rit(RIG *rig, vfo_t vfo, shortfreq_t *rit)
 		if (caps->get_rit == NULL)
 			return -RIG_ENAVAIL;
 
-		if ((caps->targetable_vfo&RIG_TARGETABLE_ALL) || 
+		if ((caps->targetable_vfo&RIG_TARGETABLE_PURE) || 
 						vfo == RIG_VFO_CURR || vfo == rig->state.current_vfo)
 			return caps->get_rit(rig, vfo, rit);
 
@@ -1839,7 +1900,7 @@ int rig_get_rit(RIG *rig, vfo_t vfo, shortfreq_t *rit)
  * \sa rig_get_xit()
  */
 
-int rig_set_xit(RIG *rig, vfo_t vfo, shortfreq_t xit)
+int HAMLIB_API rig_set_xit(RIG *rig, vfo_t vfo, shortfreq_t xit)
 {
 		const struct rig_caps *caps;
 		int retcode;
@@ -1853,7 +1914,7 @@ int rig_set_xit(RIG *rig, vfo_t vfo, shortfreq_t xit)
 		if (caps->set_xit == NULL)
 			return -RIG_ENAVAIL;
 
-		if ((caps->targetable_vfo&RIG_TARGETABLE_ALL) || 
+		if ((caps->targetable_vfo&RIG_TARGETABLE_PURE) || 
 						vfo == RIG_VFO_CURR || vfo == rig->state.current_vfo)
 			return caps->set_xit(rig, vfo, xit);
 
@@ -1884,7 +1945,7 @@ int rig_set_xit(RIG *rig, vfo_t vfo, shortfreq_t xit)
  * \sa rig_set_xit()
  */
 
-int rig_get_xit(RIG *rig, vfo_t vfo, shortfreq_t *xit)
+int HAMLIB_API rig_get_xit(RIG *rig, vfo_t vfo, shortfreq_t *xit)
 {
 		const struct rig_caps *caps;
 		int retcode;
@@ -1898,7 +1959,7 @@ int rig_get_xit(RIG *rig, vfo_t vfo, shortfreq_t *xit)
 		if (caps->get_xit == NULL)
 			return -RIG_ENAVAIL;
 
-		if ((caps->targetable_vfo&RIG_TARGETABLE_ALL) || 
+		if ((caps->targetable_vfo&RIG_TARGETABLE_PURE) || 
 						vfo == RIG_VFO_CURR || vfo == rig->state.current_vfo)
 			return caps->get_xit(rig, vfo, xit);
 
@@ -1931,7 +1992,7 @@ int rig_get_xit(RIG *rig, vfo_t vfo, shortfreq_t *xit)
  * \sa rig_get_ts()
  */
 
-int rig_set_ts(RIG *rig, vfo_t vfo, shortfreq_t ts)
+int HAMLIB_API rig_set_ts(RIG *rig, vfo_t vfo, shortfreq_t ts)
 {
 		const struct rig_caps *caps;
 		int retcode;
@@ -1945,7 +2006,7 @@ int rig_set_ts(RIG *rig, vfo_t vfo, shortfreq_t ts)
 		if (caps->set_ts == NULL)
 			return -RIG_ENAVAIL;
 
-		if ((caps->targetable_vfo&RIG_TARGETABLE_ALL) || 
+		if ((caps->targetable_vfo&RIG_TARGETABLE_PURE) || 
 						vfo == RIG_VFO_CURR || vfo == rig->state.current_vfo)
 			return caps->set_ts(rig, vfo, ts);
 
@@ -1976,7 +2037,7 @@ int rig_set_ts(RIG *rig, vfo_t vfo, shortfreq_t ts)
  * \sa rig_set_ts()
  */
 
-int rig_get_ts(RIG *rig, vfo_t vfo, shortfreq_t *ts)
+int HAMLIB_API rig_get_ts(RIG *rig, vfo_t vfo, shortfreq_t *ts)
 {
 		const struct rig_caps *caps;
 		int retcode;
@@ -1990,7 +2051,7 @@ int rig_get_ts(RIG *rig, vfo_t vfo, shortfreq_t *ts)
 		if (caps->get_ts == NULL)
 			return -RIG_ENAVAIL;
 
-		if ((caps->targetable_vfo&RIG_TARGETABLE_ALL) || 
+		if ((caps->targetable_vfo&RIG_TARGETABLE_PURE) || 
 						vfo == RIG_VFO_CURR || vfo == rig->state.current_vfo)
 			return caps->get_ts(rig, vfo, ts);
 
@@ -2025,7 +2086,7 @@ int rig_get_ts(RIG *rig, vfo_t vfo, shortfreq_t *ts)
  * \sa rig_get_ant()
  */
 
-int rig_set_ant(RIG *rig, vfo_t vfo, ant_t ant)
+int HAMLIB_API rig_set_ant(RIG *rig, vfo_t vfo, ant_t ant)
 {
 		const struct rig_caps *caps;
 		int retcode;
@@ -2039,7 +2100,7 @@ int rig_set_ant(RIG *rig, vfo_t vfo, ant_t ant)
 		if (caps->set_ant == NULL)
 			return -RIG_ENAVAIL;
 
-		if ((caps->targetable_vfo&RIG_TARGETABLE_ALL) || 
+		if ((caps->targetable_vfo&RIG_TARGETABLE_PURE) || 
 						vfo == RIG_VFO_CURR || vfo == rig->state.current_vfo)
 			return caps->set_ant(rig, vfo, ant);
 
@@ -2070,7 +2131,7 @@ int rig_set_ant(RIG *rig, vfo_t vfo, ant_t ant)
  * \sa rig_set_ant()
  */
 
-int rig_get_ant(RIG *rig, vfo_t vfo, ant_t *ant)
+int HAMLIB_API rig_get_ant(RIG *rig, vfo_t vfo, ant_t *ant)
 {
 		const struct rig_caps *caps;
 		int retcode;
@@ -2084,7 +2145,7 @@ int rig_get_ant(RIG *rig, vfo_t vfo, ant_t *ant)
 		if (caps->get_ant == NULL)
 			return -RIG_ENAVAIL;
 
-		if ((caps->targetable_vfo&RIG_TARGETABLE_ALL) || 
+		if ((caps->targetable_vfo&RIG_TARGETABLE_PURE) || 
 						vfo == RIG_VFO_CURR || vfo == rig->state.current_vfo)
 			return caps->get_ant(rig, vfo, ant);
 
@@ -2122,7 +2183,7 @@ int rig_get_ant(RIG *rig, vfo_t vfo, ant_t *ant)
  *
  * \sa rig_mW2power()
  */
-int rig_power2mW(RIG *rig, unsigned int *mwpower, float power, freq_t freq, rmode_t mode)
+int HAMLIB_API rig_power2mW(RIG *rig, unsigned int *mwpower, float power, freq_t freq, rmode_t mode)
 {
 		const freq_range_t *txrange;
 
@@ -2163,7 +2224,7 @@ int rig_power2mW(RIG *rig, unsigned int *mwpower, float power, freq_t freq, rmod
  *
  * \sa rig_power2mW()
  */
-int rig_mW2power(RIG *rig, float *power, unsigned int mwpower, freq_t freq, rmode_t mode)
+int HAMLIB_API rig_mW2power(RIG *rig, float *power, unsigned int mwpower, freq_t freq, rmode_t mode)
 {
 		const freq_range_t *txrange;
 
@@ -2201,7 +2262,7 @@ int rig_mW2power(RIG *rig, float *power, unsigned int mwpower, freq_t freq, rmod
  * has been sucessful, otherwise a negative value if an error occured.
  *
  */
-shortfreq_t rig_get_resolution(RIG *rig, rmode_t mode)
+shortfreq_t HAMLIB_API rig_get_resolution(RIG *rig, rmode_t mode)
 {
 		const struct rig_state *rs;
 		int i;
@@ -2236,7 +2297,7 @@ shortfreq_t rig_get_resolution(RIG *rig, rmode_t mode)
  * \sa rig_get_powerstat()
  */
 
-int rig_set_powerstat(RIG *rig, powerstat_t status)
+int HAMLIB_API rig_set_powerstat(RIG *rig, powerstat_t status)
 {
 		if (CHECK_RIG_ARG(rig))
 			return -RIG_EINVAL;
@@ -2262,7 +2323,7 @@ int rig_set_powerstat(RIG *rig, powerstat_t status)
  * \sa rig_set_powerstat()
  */
 
-int rig_get_powerstat(RIG *rig, powerstat_t *status)
+int HAMLIB_API rig_get_powerstat(RIG *rig, powerstat_t *status)
 {
 		if (CHECK_RIG_ARG(rig) || !status)
 			return -RIG_EINVAL;
@@ -2288,7 +2349,7 @@ int rig_get_powerstat(RIG *rig, powerstat_t *status)
  *
  */
 
-int rig_reset(RIG *rig, reset_t reset)
+int HAMLIB_API rig_reset(RIG *rig, reset_t reset)
 {
 		if (CHECK_RIG_ARG(rig))
 			return -RIG_EINVAL;
@@ -2299,8 +2360,8 @@ int rig_reset(RIG *rig, reset_t reset)
 		return rig->caps->reset(rig, reset);
 }
 
-extern int rig_probe_first(port_t *p);
-extern int rig_probe_all_backends(port_t *p, rig_probe_func_t cfunc, rig_ptr_t data);
+extern int rig_probe_first(hamlib_port_t *p);
+extern int rig_probe_all_backends(hamlib_port_t *p, rig_probe_func_t cfunc, rig_ptr_t data);
 /**
  * \brief try to guess a rig
  * \param port		A pointer describing a port linking the host to the rig
@@ -2315,7 +2376,7 @@ extern int rig_probe_all_backends(port_t *p, rig_probe_func_t cfunc, rig_ptr_t d
  * \return the rig model id according to the rig_model_t type if found, 
  * otherwise RIG_MODEL_NONE if unable to determine rig model.
  */
-rig_model_t rig_probe(port_t *port)
+rig_model_t HAMLIB_API rig_probe(hamlib_port_t *port)
 {
 	if (!port)
 		return RIG_MODEL_NONE;
@@ -2340,7 +2401,7 @@ rig_model_t rig_probe(port_t *port)
  * a negative value if an error occured (in which case, cause is 
  * set appropriately).
  */
-int rig_probe_all(port_t *port, rig_probe_func_t cfunc, rig_ptr_t data)
+int HAMLIB_API rig_probe_all(hamlib_port_t *port, rig_probe_func_t cfunc, rig_ptr_t data)
 {
 	if (!port)
 		return -RIG_EINVAL;
@@ -2364,7 +2425,7 @@ int rig_probe_all(port_t *port, rig_probe_func_t cfunc, rig_ptr_t data)
  *
  * \sa rig_vfo_op()
  */
-vfo_op_t rig_has_vfo_op(RIG *rig, vfo_op_t op)
+vfo_op_t HAMLIB_API rig_has_vfo_op(RIG *rig, vfo_op_t op)
 {
 		if (!rig || !rig->caps)
 				return 0;
@@ -2388,7 +2449,7 @@ vfo_op_t rig_has_vfo_op(RIG *rig, vfo_op_t op)
  * \sa rig_has_vfo_op()
  */
 
-int rig_vfo_op(RIG *rig, vfo_t vfo, vfo_op_t op)
+int HAMLIB_API rig_vfo_op(RIG *rig, vfo_t vfo, vfo_op_t op)
 {
 		const struct rig_caps *caps;
 		int retcode;
@@ -2402,7 +2463,7 @@ int rig_vfo_op(RIG *rig, vfo_t vfo, vfo_op_t op)
 		if (caps->vfo_op == NULL || !rig_has_vfo_op(rig,op))
 			return -RIG_ENAVAIL;
 
-		if ((caps->targetable_vfo&RIG_TARGETABLE_ALL) ||
+		if ((caps->targetable_vfo&RIG_TARGETABLE_PURE) ||
 				vfo == RIG_VFO_CURR || vfo == rig->state.current_vfo)
 			return caps->vfo_op(rig, vfo, op);
 
@@ -2434,7 +2495,7 @@ int rig_vfo_op(RIG *rig, vfo_t vfo, vfo_op_t op)
  *
  * \sa rig_scan()
  */
-scan_t rig_has_scan(RIG *rig, scan_t scan)
+scan_t HAMLIB_API rig_has_scan(RIG *rig, scan_t scan)
 {
 		if (!rig || !rig->caps)
 				return 0;
@@ -2459,7 +2520,7 @@ scan_t rig_has_scan(RIG *rig, scan_t scan)
  * \sa rig_has_scan()
  */
 
-int rig_scan(RIG *rig, vfo_t vfo, scan_t scan, int ch)
+int HAMLIB_API rig_scan(RIG *rig, vfo_t vfo, scan_t scan, int ch)
 {
 		const struct rig_caps *caps;
 		int retcode;
@@ -2474,7 +2535,7 @@ int rig_scan(RIG *rig, vfo_t vfo, scan_t scan, int ch)
 						(scan!=RIG_SCAN_STOP && !rig_has_scan(rig, scan)))
 			return -RIG_ENAVAIL;
 
-		if ((caps->targetable_vfo&RIG_TARGETABLE_ALL) ||
+		if ((caps->targetable_vfo&RIG_TARGETABLE_PURE) ||
 				vfo == RIG_VFO_CURR || vfo == rig->state.current_vfo)
 			return caps->scan(rig, vfo, scan, ch);
 
@@ -2505,7 +2566,7 @@ int rig_scan(RIG *rig, vfo_t vfo, scan_t scan, int ch)
  *
  */
 
-int rig_send_dtmf(RIG *rig, vfo_t vfo, const char *digits)
+int HAMLIB_API rig_send_dtmf(RIG *rig, vfo_t vfo, const char *digits)
 {
 		const struct rig_caps *caps;
 		int retcode;
@@ -2519,7 +2580,7 @@ int rig_send_dtmf(RIG *rig, vfo_t vfo, const char *digits)
 		if (caps->send_dtmf == NULL)
 			return -RIG_ENAVAIL;
 
-		if ((caps->targetable_vfo&RIG_TARGETABLE_ALL) ||
+		if ((caps->targetable_vfo&RIG_TARGETABLE_PURE) ||
 				vfo == RIG_VFO_CURR || vfo == rig->state.current_vfo)
 			return caps->send_dtmf(rig, vfo, digits);
 
@@ -2551,7 +2612,7 @@ int rig_send_dtmf(RIG *rig, vfo_t vfo, const char *digits)
  *
  */
 
-int rig_recv_dtmf(RIG *rig, vfo_t vfo, char *digits, int *length)
+int HAMLIB_API rig_recv_dtmf(RIG *rig, vfo_t vfo, char *digits, int *length)
 {
 		const struct rig_caps *caps;
 		int retcode;
@@ -2565,7 +2626,7 @@ int rig_recv_dtmf(RIG *rig, vfo_t vfo, char *digits, int *length)
 		if (caps->recv_dtmf == NULL)
 			return -RIG_ENAVAIL;
 
-		if ((caps->targetable_vfo&RIG_TARGETABLE_ALL) ||
+		if ((caps->targetable_vfo&RIG_TARGETABLE_PURE) ||
 				vfo == RIG_VFO_CURR || vfo == rig->state.current_vfo)
 			return caps->recv_dtmf(rig, vfo, digits, length);
 
@@ -2596,7 +2657,7 @@ int rig_recv_dtmf(RIG *rig, vfo_t vfo, char *digits, int *length)
  *
  */
 
-int rig_send_morse(RIG *rig, vfo_t vfo, const char *msg)
+int HAMLIB_API rig_send_morse(RIG *rig, vfo_t vfo, const char *msg)
 {
 		const struct rig_caps *caps;
 		int retcode;
@@ -2610,7 +2671,7 @@ int rig_send_morse(RIG *rig, vfo_t vfo, const char *msg)
 		if (caps->send_morse == NULL)
 			return -RIG_ENAVAIL;
 
-		if ((caps->targetable_vfo&RIG_TARGETABLE_ALL) ||
+		if ((caps->targetable_vfo&RIG_TARGETABLE_PURE) ||
 				vfo == RIG_VFO_CURR || vfo == rig->state.current_vfo)
 			return caps->send_morse(rig, vfo, msg);
 
@@ -2641,7 +2702,7 @@ int rig_send_morse(RIG *rig, vfo_t vfo, const char *msg)
  *
  */
 const freq_range_t *
-rig_get_range(const freq_range_t range_list[], freq_t freq, rmode_t mode)
+HAMLIB_API rig_get_range(const freq_range_t range_list[], freq_t freq, rmode_t mode)
 {
 	int i;
 
@@ -2669,7 +2730,7 @@ rig_get_range(const freq_range_t range_list[], freq_t freq, rmode_t mode)
  * if the operation has been sucessful, otherwise NULL if an error occured
  * or get_info not part of capabilities.
  */
-const char* rig_get_info(RIG *rig)
+const char* HAMLIB_API rig_get_info(RIG *rig)
 {
 		if (CHECK_RIG_ARG(rig))
 			return NULL;
