@@ -1,11 +1,11 @@
 /*
- * rigctl.c - (C) Stephane Fillod 2000-2003
+ * rigctl.c - (C) Stephane Fillod 2000-2005
  *
  * This program test/control a radio using Hamlib.
  * It takes commands in interactive mode as well as 
  * from command line options.
  *
- * $Id: rigctl.c,v 1.50 2004/05/17 21:09:45 fillods Exp $  
+ * $Id: rigctl.c,v 1.55 2005/03/13 17:55:47 fillods Exp $  
  *
  *
  * This program is free software; you can redistribute it and/or
@@ -41,12 +41,6 @@
 #include "misc.h"
 #include "sprintflst.h"
 
-#ifdef _WIN32
-#define LLFMT "L"
-#else
-#define LLFMT "ll"
-#endif
-
 #define MAXNAMSIZ 32
 #define MAXNBOPT 100	/* max number of different options */
 
@@ -59,6 +53,7 @@
 #define ARG_OUT3 0x20
 #define ARG_IN4  0x40
 #define ARG_OUT4 0x80
+#define ARG_IN_LINE 0x4000
 #define ARG_NOVFO 0x8000
 
 #define ARG_IN  (ARG_IN1|ARG_IN2|ARG_IN3|ARG_IN4)
@@ -147,6 +142,9 @@ declare_proto_rig(dump_caps);
 declare_proto_rig(set_ant);
 declare_proto_rig(get_ant);
 declare_proto_rig(reset);
+declare_proto_rig(send_morse);
+declare_proto_rig(set_powerstat);
+declare_proto_rig(get_powerstat);
 
 
 
@@ -209,6 +207,9 @@ struct test_table test_list[] = {
 		{ 0x83, "set_ant", set_ant, ARG_IN, "Antenna" },
 		{ 0x84, "get_ant", get_ant, ARG_OUT, "Antenna" },
 		{ 0x85, "reset", reset, ARG_IN, "Reset" },
+		{ 0x86, "send_morse", send_morse, ARG_IN|ARG_IN_LINE, "Morse" },
+		{ 0x87, "set_powerstat", set_powerstat, ARG_IN|ARG_NOVFO, "Status" },
+		{ 0x88, "get_powerstat", get_powerstat, ARG_OUT|ARG_NOVFO, "Status" },
 		{ 0x00, "", NULL },
 
 };
@@ -590,7 +591,25 @@ int main (int argc, char *argv[])
 					vfo = rig_parse_vfo(argv[optind++]);
 				}
 			}
-			if ((cmd_entry->flags & ARG_IN1) && cmd_entry->arg1) {
+			if ((cmd_entry->flags & ARG_IN_LINE) && 
+					(cmd_entry->flags & ARG_IN1) && cmd_entry->arg1) {
+				if (interactive) {
+					char *nl;
+					printf("%s: ", cmd_entry->arg1);
+					fgets(arg1, MAXARGSZ, stdin);
+					nl = strchr(arg1, 0xa);
+					if (nl) *nl = '\0';	/* chomp */
+					p1 = arg1;
+				} else {
+					if (!argv[optind]) {
+						fprintf(stderr, "Invalid arg for command '%s'\n", 
+									cmd_entry->name);
+						exit(2);
+					}
+					p1 = argv[optind++];
+				}
+			} else 
+			  if ((cmd_entry->flags & ARG_IN1) && cmd_entry->arg1) {
 				if (interactive) {
 					printf("%s: ", cmd_entry->arg1);
 					scanfc("%s", arg1);
@@ -661,11 +680,11 @@ void usage_rig()
 		printf("Commands (may not be available for this rig):\n");
 		for (i=0; test_list[i].cmd != 0; i++) {
 			printf("%c: %-16s(", isprint(test_list[i].cmd)?test_list[i].cmd:'?', test_list[i].name);
-			if (test_list[i].arg1)
+			if (test_list[i].arg1 && (test_list[i].flags&ARG_IN1))
 					printf("%s", test_list[i].arg1);
-			if (test_list[i].arg2)
+			if (test_list[i].arg2 && (test_list[i].flags&ARG_IN2))
 					printf(",%s", test_list[i].arg2);
-			if (test_list[i].arg3)
+			if (test_list[i].arg3 && (test_list[i].flags&ARG_IN3))
 					printf(",%s", test_list[i].arg3);
 			printf(")  \t");
 
@@ -795,7 +814,7 @@ declare_proto_rig(set_freq)
 {
 		freq_t freq;
 
-		sscanf(arg1, "%"FREQFMT, &freq);
+		sscanf(arg1, "%"SCNfreq, &freq);
 		return rig_set_freq(rig, vfo, freq);
 }
 
@@ -809,7 +828,7 @@ declare_proto_rig(get_freq)
 				return status;
 		if (interactive)
 			printf("%s: ", cmd->arg1); /* i.e. "Frequency" */
-		printf("%lld\n", (long long)freq);
+		printf("%"PRIll"\n", (long long)freq);
 		return status;
 }
 
@@ -1032,7 +1051,7 @@ declare_proto_rig(set_split_freq)
 {
 		freq_t txfreq;
 
-		sscanf(arg1, "%"FREQFMT, &txfreq);
+		sscanf(arg1, "%"SCNfreq, &txfreq);
 		return rig_set_split_freq(rig, vfo, txfreq);
 }
 
@@ -1047,7 +1066,7 @@ declare_proto_rig(get_split_freq)
 				return status;
 		if (interactive)
 			printf("%s: ", cmd->arg1);
-		printf("%lld\n", (long long)txfreq);
+		printf("%"PRIll"\n", (long long)txfreq);
 		return status;
 }
 
@@ -1143,7 +1162,7 @@ declare_proto_rig(power2mW)
 		printf("Power [0.0 .. 1.0]: ");
 		scanf("%f", &power);
 		printf("Frequency: ");
-		scanf("%"FREQFMT, &freq);
+		scanf("%"SCNfreq, &freq);
 		printf("Mode: ");
 		scanf("%d", &mode);
 		status = rig_power2mW(rig, &mwp, power, freq, mode);
@@ -1447,7 +1466,7 @@ declare_proto_rig(get_channel)
 
 static int myfreq_event(RIG *rig, vfo_t vfo, freq_t freq, rig_ptr_t arg)
 {
-	printf("Event: freq changed to %lliHz on %s\n", (long long)freq, rig_strvfo(vfo));
+	printf("Event: freq changed to %"PRIll"Hz on %s\n", (long long)freq, rig_strvfo(vfo));
 	return 0;
 }
 
@@ -1628,4 +1647,30 @@ declare_proto_rig(reset)
 	return rig_reset(rig, reset);
 }
 
+declare_proto_rig(send_morse)
+{
+	return rig_send_morse(rig, vfo, arg1);
+}
+
+declare_proto_rig(set_powerstat)
+{
+	powerstat_t stat;
+
+	sscanf(arg1, "%d", &stat);
+	return rig_set_powerstat(rig, stat);
+}
+
+declare_proto_rig(get_powerstat)
+{
+	int status;
+	powerstat_t stat;
+
+	status = rig_get_powerstat(rig, &stat);
+	if (status != RIG_OK)
+		return status;
+	if (interactive)
+		printf("%s: ", cmd->arg1);
+	printf("%d\n", stat);
+	return status;
+}
 
