@@ -13,7 +13,7 @@
  * The starting point for this code was Frank's ft847 implementation.
  *
  *
- *    $Id: ft857.c,v 1.4 2003/10/01 19:34:08 fillods Exp $  
+ *    $Id: ft857.c,v 1.7 2005/01/25 00:21:58 fillods Exp $  
  *
  *
  *  This library is free software; you can redistribute it and/or
@@ -69,6 +69,10 @@
 #include <string.h>  	/* String function definitions */
 #include <unistd.h>  	/* UNIX standard function definitions */
 
+#ifdef HAVE_SYS_TIME_H
+#include <sys/time.h>
+#endif
+
 #include "hamlib/rig.h"
 #include "serial.h"
 #include "yaesu.h"
@@ -161,7 +165,7 @@ const struct rig_caps ft857_caps = {
   .retry = 		0, 
   .has_get_func =       RIG_FUNC_NONE,
   .has_set_func = 	RIG_FUNC_LOCK | RIG_FUNC_TONE | RIG_FUNC_TSQL, 
-  .has_get_level = 	RIG_LEVEL_STRENGTH | RIG_LEVEL_RFPOWER | RIG_LEVEL_SQLSTAT,
+  .has_get_level = 	RIG_LEVEL_STRENGTH | RIG_LEVEL_RFPOWER,
   .has_set_level = 	RIG_LEVEL_NONE,
   .has_get_parm = 	RIG_PARM_NONE,
   .has_set_parm = 	RIG_PARM_NONE,
@@ -523,9 +527,6 @@ static int ft857_get_smeter_level(RIG *rig, value_t *val)
 
 int ft857_get_level(RIG *rig, vfo_t vfo, setting_t level, value_t *val)
 {
-  dcd_t dcd;
-  int n;
-
   if (vfo != RIG_VFO_CURR)
     return -RIG_ENTARGET;
 
@@ -536,12 +537,6 @@ int ft857_get_level(RIG *rig, vfo_t vfo, setting_t level, value_t *val)
   case RIG_LEVEL_RFPOWER:
     return ft857_get_pometer_level(rig, val);
 
-  case RIG_LEVEL_SQLSTAT:
-    if ((n = ft857_get_dcd(rig, vfo, &dcd)) < 0)
-      return n;
-    val->i = dcd;
-    break;
-
   default:
     return -RIG_EINVAL;
   }
@@ -638,7 +633,7 @@ int ft857_set_freq(RIG *rig, vfo_t vfo, freq_t freq)
   if (vfo != RIG_VFO_CURR)
     return -RIG_ENTARGET;
 
-  rig_debug(RIG_DEBUG_VERBOSE,"ft857: requested freq = %lli Hz\n", freq);
+  rig_debug(RIG_DEBUG_VERBOSE,"ft857: requested freq = %"PRIfreq" Hz\n", freq);
 
   /* fill in the frequency */
   to_bcd_be(data, (freq + 5) / 10, 8);
