@@ -2,14 +2,14 @@
  * hamlib - (C) Frank Singleton 2000 (javabear at users.sourceforge.net)
  *
  * ft900.c - (C) Frank Singleton 2000 (javabear at users.sourceforge.net)
- *           (C) Stephane Fillod 2002, 2003 (fillods at users.sourceforge.net)
+ *           (C) Stephane Fillod 2002-2005 (fillods at users.sourceforge.net)
  *           (C) Nate Bargmann 2002, 2003 (n0nb at arrl.net)
  *
  * This shared library provides an API for communicating
  * via serial interface to an FT-900 using the "CAT" interface
  *
  *
- * $Id: ft900.c,v 1.1 2003/04/14 22:17:44 n0nb Exp $
+ * $Id: ft900.c,v 1.3 2005/04/03 18:57:28 fillods Exp $
  *
  *
  *  This library is free software; you can redistribute it and/or
@@ -258,10 +258,6 @@ const struct rig_caps ft900_caps = {
   .get_ptt =            ft900_get_ptt,
   .set_split_vfo =          ft900_set_split_vfo,
   .get_split_vfo =          ft900_get_split_vfo,
-//  .set_split_freq =     ft900_set_split_freq,
-//  .get_split_freq =     ft900_get_split_freq,
-//  .set_split_mode =     ft900_set_split_mode,
-//  .get_split_mode =     ft900_get_split_mode,
   .set_rit =            ft900_set_rit,
   .get_rit =            ft900_get_rit,
   .set_func =           ft900_set_func,
@@ -405,7 +401,7 @@ static int ft900_set_freq(RIG *rig, vfo_t vfo, freq_t freq) {
   priv = (struct ft900_priv_data *)rig->state.priv;
 
   rig_debug(RIG_DEBUG_TRACE, "%s: passed vfo = 0x%02x\n", __func__, vfo);
-  rig_debug(RIG_DEBUG_TRACE, "%s: passed freq = %lli Hz\n", __func__, freq);
+  rig_debug(RIG_DEBUG_TRACE, "%s: passed freq = %"PRIfreq" Hz\n", __func__, freq);
 
   if (vfo == RIG_VFO_CURR) {
     vfo = priv->current_vfo;    /* from previous vfo cmd */
@@ -487,7 +483,7 @@ static int ft900_get_freq(RIG *rig, vfo_t vfo, freq_t *freq) {
   f = ((((p[0]<<8) + p[1])<<8) + p[2]) * 10;
 
   rig_debug(RIG_DEBUG_TRACE,
-            "%s: freq = %lli Hz for vfo 0x%02x\n", __func__, f, vfo);
+            "%s: freq = %"PRIfreq" Hz for vfo 0x%02x\n", __func__, f, vfo);
 
   *freq = f;                    /* return displayed frequency */
 
@@ -1060,105 +1056,6 @@ static int ft900_get_split_vfo(RIG *rig, vfo_t vfo, split_t *split, vfo_t *tx_vf
     return RIG_OK;
 }
 
-
-/*
- * set the '900 split TX freq
- *
- * Right now this is just a pass-through function and depends on the
- * user app to "know" which VFO to set.  Does this need to determine
- * the split direction and set accordingly?
- */
-
-static int ft900_set_split_freq(RIG *rig, vfo_t vfo, freq_t tx_freq) {
-  int err;
-
-  rig_debug(RIG_DEBUG_VERBOSE, "%s called\n", __func__);
-
-  if (!rig)
-    return -RIG_EINVAL;
-
-  err = ft900_set_freq(rig, vfo, tx_freq);
-  if (err != RIG_OK)
-    return err;
-
-  return RIG_OK;
-}
-
-
-/*
- * get the '900 split TX freq
- *
- * Right now this is just a pass-through function and depends on the
- * user app to "know" which VFO to set.  Does this need to determine
- * the split direction and set accordingly?
- */
-
-static int ft900_get_split_freq(RIG *rig, vfo_t vfo, freq_t *tx_freq) {
-  int err;
-
-  rig_debug(RIG_DEBUG_VERBOSE, "%s called\n", __func__);
-
-  if (!rig)
-    return -RIG_EINVAL;
-
-  err = ft900_get_freq(rig, vfo, tx_freq);
-  if (err != RIG_OK)
-    return err;
-
-  return RIG_OK;
-}
-
-
-/*
- * set the '900 split TX mode
- *
- * Right now this is just a pass-through function and depends on the
- * user app to "know" which VFO to set.  Does this need to determine
- * the split direction and set accordingly?
- */
-
-static int ft900_set_split_mode(RIG *rig, vfo_t vfo, rmode_t tx_mode,
-                                pbwidth_t tx_width) {
-  int err;
-
-  rig_debug(RIG_DEBUG_VERBOSE, "%s called\n", __func__);
-
-  if (!rig)
-    return -RIG_EINVAL;
-
-  err = ft900_set_mode(rig, vfo, tx_mode, tx_width);
-  if (err != RIG_OK)
-    return err;
-
-  return RIG_OK;
-}
-
-
-/*
- * get the '900 split TX mode
- *
- * Right now this is just a pass-through function and depends on the
- * user app to "know" which VFO to set.  Does this need to determine
- * the split direction and set accordingly?
- */
-
-static int ft900_get_split_mode(RIG *rig, vfo_t vfo, rmode_t *tx_mode,
-                                pbwidth_t *tx_width) {
-  int err;
-
-  rig_debug(RIG_DEBUG_VERBOSE, "%s called\n", __func__);
-
-  if (!rig)
-    return -RIG_EINVAL;
-
-  err = ft900_get_mode(rig, vfo, tx_mode, tx_width);
-  if (err != RIG_OK)
-    return err;
-
-  return RIG_OK;
-}
-
-
 /*
  * rig_set_rit
  *
@@ -1627,7 +1524,7 @@ static int ft900_send_dial_freq(RIG *rig, unsigned char ci, freq_t freq) {
     return -RIG_EINVAL;
 
   rig_debug(RIG_DEBUG_TRACE, "%s: passed ci = %i\n", __func__, ci);
-  rig_debug(RIG_DEBUG_TRACE, "%s: passed freq = %lli Hz\n", __func__, freq);
+  rig_debug(RIG_DEBUG_TRACE, "%s: passed freq = %"PRIfreq" Hz\n", __func__, freq);
 
   priv = (struct ft900_priv_data *)rig->state.priv;
   if (priv->pcs[ci].ncomp) {
@@ -1645,7 +1542,7 @@ static int ft900_send_dial_freq(RIG *rig, unsigned char ci, freq_t freq) {
   to_bcd(priv->p_cmd, freq/10, FT900_BCD_DIAL);
 
   rig_debug(RIG_DEBUG_TRACE,
-            "%s: requested freq after conversion = %lli Hz\n",
+            "%s: requested freq after conversion = %"PRIll" Hz\n",
             __func__, from_bcd(priv->p_cmd, FT900_BCD_DIAL)* 10);
 
   err = write_block(&rig_s->rigport, (unsigned char *) &priv->p_cmd,
